
//
// bidwatcher
// copyright (c) 1999, 2000
// Trent McNair (trent@rmci.net)
// Tom McNair  (tmcnair@cyberhighway.net)
// Wayne Schlitt (wayne@midwestcs.com)
// 
// use of this code is restricted to the terms
// of the GPL, which should have been included in this
// distribution. If not, see www.gnu.org/copyleft/gpl.html.
// Here is the short version:
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/signal.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <stdio.h>
#include <netdb.h>
#include <fstream.h>
#include <strstream.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include "qtbidwatcher.h"
#include "images.cpp"
#include "netstuff.cpp"
#include "helpers.cpp"

bool configOpen;
bool getNumOpen;

//////////////////////////////////////////////////////////////////////////////
////////////////----------MyListBoxItem Class------------/////////////////////
//////////////////////////////////////////////////////////////////////////////
void MyListBoxItem::paint( QPainter *p )
{

   p->drawPixmap( 1, 0, pm );
   p->setPen(textColor);
   QFontMetrics fm = p->fontMetrics();
   int yPos;                       // vertical text position
   if ( pm.height() < fm.height() )
      yPos = fm.ascent() + fm.leading()/2;
   else
      yPos = pm.height()/2 - fm.height()/2 + fm.ascent();
   p->drawText( pm.width() + 3, yPos - 0, text() );
}

int MyListBoxItem::height(const QListBox *lb ) const
{
   return QMAX( pm.height() - 1 , lb->fontMetrics().lineSpacing() + 1 );
}

int MyListBoxItem::width(const QListBox *lb ) const
{
   return pm.width() + lb->fontMetrics().width( text() ) + 6;
}
//////////////////////////////////////////////////////////////////////////////
///////////----------ListBox Class-------------////////////////////////////
//////////////////////////////////////////////////////////////////////////////
NewListBox::NewListBox(QWidget * parent=0, const char * name=0, WFlags f=0 )
           : QListBox(parent, name, f)
{
}
void NewListBox::mousePressEvent(QMouseEvent * thisEvent)
{
   QListBox::mousePressEvent(thisEvent);
   if (thisEvent->button() == RightButton ) emit RightClicked();
   if (thisEvent->button() == MidButton ) emit MiddleClicked();
}
///////////////////////////////////////////////////////////////////////////////
////////////----------BidWindow Class---------/////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
BidWindow::BidWindow( QWidget * parent=0, const char * name=0, bool modal=FALSE,
                 WFlags f=0 )
        : QDialog(parent, name, modal, f)
{
   doSnipe = FALSE;
   setFixedSize(373,235);
   setGeometry( 10,10,650,91);

   QColorGroup redColorGroup(red,white,white,white,white,red,white);
   QColorGroup grayColorGroup(white,darkGray,darkGray,darkGray,darkGray,white,darkGray);
   QColorGroup blackColorGroup(white,black,black,black,black,white,black);
   grayPalette = QPalette(grayColorGroup,grayColorGroup,grayColorGroup);
   blackPalette = QPalette(blackColorGroup,blackColorGroup,blackColorGroup);
   redPalette = QPalette(redColorGroup,redColorGroup,redColorGroup);

   DETAIL_TIMER = new QTimer( this );
   CHECK_PTR( DETAIL_TIMER );
   connect( DETAIL_TIMER, SIGNAL(timeout()), SLOT(timerUp()) );

   itemLabel = new QLabel( "", this );
   CHECK_PTR( itemLabel );
   itemLabel->setGeometry(5,5,363,22);
   itemLabel->setAlignment(AlignHCenter | AlignVCenter);
   itemLabel->setPalette(grayPalette);
   itemLabel->setFont(QFont("helvetica", 17, QFont::Bold));

   descriptionLabel = new QLabel( "", this );
   CHECK_PTR( descriptionLabel );
   descriptionLabel->setGeometry(5,27,363,18);
   descriptionLabel->setAlignment(AlignHCenter | AlignVCenter);
   descriptionLabel->setPalette(grayPalette);
   descriptionLabel->setFont(QFont("helvetica", 12, QFont::Normal));
   
   sideBarLabel = new QLabel( "", this );
   CHECK_PTR( sideBarLabel );
   sideBarLabel->setGeometry(5,45,16,185);
   sideBarLabel->setPalette(grayPalette);

   priceLabel = new QLabel( "", this );
   CHECK_PTR( priceLabel );
   priceLabel->setGeometry(21,45,347,20);
   priceLabel->setAlignment(AlignHCenter | AlignVCenter);
   priceLabel->setPalette(redPalette);
   priceLabel->setFont(QFont("helvetica", 17, QFont::Bold));
      
   categoryLabel = new QLabel( "", this );
   CHECK_PTR( categoryLabel );
   categoryLabel->setGeometry(26,70,342,160);
   categoryLabel->setAlignment(AlignRight);
   categoryLabel->setPalette(blackPalette);

   infoLabel = new QLabel( "Your MAXIMUM bid", this );
   CHECK_PTR( infoLabel );
   infoLabel->setGeometry(130,103,165,20);
   infoLabel->setAlignment(AlignLeft);
   infoLabel->setPalette(blackPalette);
   infoLabel->setFont(QFont("helvetica", 14, QFont::Bold));

   // reserve space for bid submission	// DBS
   bidLabel = new QLabel( "Dutch Auction Quantity", this );
   CHECK_PTR( bidLabel );
   bidLabel->setGeometry(130,128,165,20);
   bidLabel->setAlignment(AlignLeft);
   bidLabel->setPalette(blackPalette);
   bidLabel->setFont(QFont("helvetica", 14, QFont::Bold));

   prevSnipeLabel = new QLabel( "", this );
   CHECK_PTR( prevSnipeLabel );
   prevSnipeLabel->setGeometry(45,153,250,20);
   prevSnipeLabel->setAlignment(AlignLeft);
   prevSnipeLabel->setPalette(blackPalette);
   prevSnipeLabel->setFont(QFont("helvetica", 14, QFont::Bold));


   timeLabel = new QLabel( "", this );
   CHECK_PTR( timeLabel );
   timeLabel->setGeometry(115,75,190,14);
   timeLabel->setAlignment(AlignLeft);
   timeLabel->setPalette(blackPalette);
    
   cancelBidButton = new QPushButton(this);
   CHECK_PTR(cancelBidButton);
   cancelBidButton->setGeometry(169, 205, 55, 20);
   cancelBidButton->setText("cancel");
   connect( cancelBidButton, SIGNAL(clicked()), SLOT(CmCancel()) );
   
   bidButton = new QPushButton(this);
   CHECK_PTR(bidButton);
   bidButton->setGeometry(47, 205, 55, 20);
   bidButton->setText("bid now");
   connect( bidButton, SIGNAL(clicked()), SLOT(CmBidNow()) );
   
   snipeButton = new QPushButton(this);
   CHECK_PTR(snipeButton);
   snipeButton->setGeometry(292, 205, 55, 20);
   snipeButton->setText("snipe");
   connect( snipeButton, SIGNAL(clicked()), SLOT(CmSnipe()) );
   
   maxBid = new QLineEdit( this, "" );
   CHECK_PTR( maxBid );
   maxBid->setGeometry( 45, 100, 80, 22 ); 

   // A place to store maximum quantity to bid on   // DBS
   maxQuant = new QLineEdit( this, "" );
   CHECK_PTR( maxQuant );
   maxQuant->setGeometry( 45, 125, 80, 22 ); 
}
void BidWindow::fillBlanks(struct BidMakeup_type & bidAuction, int idx,
                            char * name, char * pass, int offset, char * CurrentBid)
{ 
   // initialize
   ebayOffset = offset;
   doSnipe = FALSE;
   cancelBidButton->setEnabled(TRUE);
   bidButton->setEnabled(TRUE);
   snipeButton->setEnabled(TRUE);
   char temp[200];
   char chTemp[20];
   char currency[10];
   theAuction = bidAuction;
   theAuctionIdx = idx;
   strcpy(userID, name);
   strcpy(userPASS, pass);


   // fill values
   if ( bidAuction.BidCount[0] == '0' ) {
      if ( bidAuction.FirstBid[0] != '0' ) {

         int max = strlen(bidAuction.FirstBid);
         int i = 0;
         while ( i < max  ) {
            temp[i] = bidAuction.FirstBid[i + 1];
            i++;
         }
      }
      else strcpy(temp, "");
   }
   else 
   strip_commas(temp);
   strip_space(temp);
   if ( bidAuction.BidCount[0] == '0' ) {

 parsebid(bidAuction.CurrentBid,temp);
	if (temp[0] == '$' ) {
                int limit = strlen(temp);
                for ( int i=0; i < limit; i++ )
                {
                        // strip the '$' off the currentBid;
                        temp[i] = temp[i + 1];
                }
		strcpy(bidAuction.CurrentBid,temp);
		}
		}

 if ( bidAuction.BidCount[0] != '0' ) calculateBidIncrement(bidAuction.CurrentBid, temp, currency);
   maxBid->setText(temp);
   strcpy(temp, "Item Number ");
   strcat(temp, theAuction.ItemNumber);
   itemLabel->setText(temp);
   descriptionLabel->setText(theAuction.Description);
   strcat(temp, theAuction.CurrentBid);
   strcpy(temp, "Current Bid : ");
   strcat(temp, theAuction.CurrentBid);
   if ( theAuction.reserveMet == 'y' ) strcat(temp, " - reserve met");
   else if ( theAuction.reserveMet == 'n' ) strcat(temp, " - reserve not met");
   else if ( theAuction.reserveMet == 'x' ) strcat(temp, " - no reserve");
   priceLabel->setText(temp);
   strcpy(temp, "Time Left : ");
   stringTimeLeft(theAuction.EndsValue, offset, chTemp);
   strcat(temp, chTemp);
   maxQuant->setText("1");	// DBS
   maxQuant->setEnabled( strcmp( theAuction.HighBidder, "Dutch Auction") == 0 );	// DBS
   bidLabel->setEnabled( strcmp( theAuction.HighBidder, "Dutch Auction") == 0 );	// DBS
   timeLabel->setText(temp);
   DETAIL_TIMER->start(1000,FALSE);     
   
   if( theAuction.isSnipe )
   {
       sprintf( temp, "Previous snipe amount: $%s", theAuction.snipeAmount);
       prevSnipeLabel->setText( temp );
   }
   else
       prevSnipeLabel->setText( "" );
	}
	void BidWindow::CmCancel()
	{
   DETAIL_TIMER->stop();
   hide();
	}

	void BidWindow::CmBidNow()
	{
   bidButton->setEnabled(FALSE);
   cancelBidButton->setEnabled(FALSE);
   snipeButton->setEnabled(FALSE);
   priceLabel->setText("Obtaining key...");
   char Buff[HUGEBUFF];
   char bidAmount[15];
   char bidQuant[15];
   char WebPage[200];
   strcpy(bidAmount, maxBid->text() );
   strcpy(bidQuant, maxQuant->text() );
   if ( !theAuction.auc_type_save ) strcpy(WebPage, "/aw-cgi/eBayISAPI.dll?MakeBid&item=");
   if ( theAuction.auc_type_save ) strcpy(WebPage, "/ebaymotors/aw-cgi/eBayISAPI.dll?MakeBid&item=");

   strcat(WebPage, theAuction.ItemNumber);
   strcat(WebPage, "&maxbid=");
   strcat(WebPage,  bidAmount);
   strcat(WebPage, "&quant=");
   strcat(WebPage, bidQuant);
   int returnVal = FetchHtml(WebPage, Buff, theAddress, HUGEBUFF, TIMEOUT);
   if ( returnVal != 1 )
   {
      if ( returnVal == 2 ){
         QMessageBox::about(this,"Error","Could not obtain key: a network error occurred.");
	}
      else if ( returnVal == 4 ){
         QMessageBox::about(this,"Error","Network timeout error");
      cancelBidButton->setEnabled(TRUE);
      bidButton->setEnabled(TRUE);
      snipeButton->setEnabled(TRUE);
      priceLabel->setText("");
      return;
	}
   }
   char lineBuff[1024];
   returnVal = ProcessBidSubmission(Buff, lineBuff);
   if ( returnVal > 1 )
   {
      if ( returnVal == 2 ){
         QMessageBox::about(this,"Error","Bid submission failed:\nBid is lower than current asking price");
	}
      else if ( returnVal == 3 ){
         QMessageBox::about(this,"Error","Bid submission failed:\nProblem with bid quantity.");
	}
      else 
	{
         QMessageBox::about(this,"Error","Bid submission failed:\nUnknown problem.");   
      cancelBidButton->setEnabled(TRUE);
      bidButton->setEnabled(TRUE);
      snipeButton->setEnabled(TRUE);
      priceLabel->setText("");
      return;
	}
   }
   // ok, eBay liked our bid submission so we can go ahead and make the bid
   char key[100];
   char * scratch;
   scratch = strtok(lineBuff, "\"");
   scratch = strtok(0,"\"");
   strcpy(key, scratch);
   if ( doSnipe )
   {
      strcpy(theAuction.snipeAmount, bidAmount);
      strcpy(theAuction.snipeQty, bidQuant);
      strcpy(theAuction.snipeKey, key);
      theAuction.isSnipe = TRUE;
      cancelBidButton->setEnabled(TRUE);
      bidButton->setEnabled(TRUE);
      snipeButton->setEnabled(TRUE);
      priceLabel->setText("");
      emit setsnipe(theAuction, theAuctionIdx);
      DETAIL_TIMER->stop();
      hide();
      return;
   }
   else
   {
      priceLabel->setText("Submitting bid...");
      // now, finalize bid with the key obtained.
	if ( !theAuction.auc_type_save ) strcpy(WebPage, "/aw-cgi/eBayISAPI.dll?AcceptBid&item=");
	if ( theAuction.auc_type_save ) strcpy(WebPage, "/ebaymotors/aw-cgi/eBayISAPI.dll?AcceptBid&item=");
      strcat(WebPage, theAuction.ItemNumber);
      strcat(WebPage, "&key=");
      strcat(WebPage, key);
      strcat(WebPage, "&userid=");
      strcat(WebPage, userID);
      strcat(WebPage, "&pass=");
      strcat(WebPage, userPASS);
      strcat(WebPage, "&maxbid=");
      strcat(WebPage,  bidAmount);
      strcat(WebPage, "&quant=");
      strcat(WebPage, bidQuant);
      returnVal = FetchHtml(WebPage, Buff, theAddress, HUGEBUFF, TIMEOUT);
      if ( returnVal != 1 )
      {
         if ( returnVal == 2 ){
            QMessageBox::about(this,"Error","Could not obtain key: a network error occurred.");
	}
         else if ( returnVal == 4 ){
            QMessageBox::about(this,"Error","Network timeout error");
         cancelBidButton->setEnabled(TRUE);
         bidButton->setEnabled(TRUE);
         snipeButton->setEnabled(TRUE);
         strcpy(WebPage, "Current bid price: ");
         strcat(WebPage, theAuction.CurrentBid);
         if ( theAuction.reserveMet == 'y' ) strcat(WebPage, " - reserve met");
         else if ( theAuction.reserveMet == 'n' ) strcat(WebPage, " - reserve not met");
         else if ( theAuction.reserveMet == 'x' ) strcat(WebPage, " - no reserve");
         priceLabel->setText("WebPage");
         return;
      }
	}
      returnVal = ProcessBid( Buff );
      if ( returnVal == 1 ){
         QMessageBox::about(this,"Bid Results","Bid completed:\nYou are the high bidder");
	}
      else if ( returnVal == 2 ){
         QMessageBox::about(this,"Bid Results","Bid completed:\nSomeone has outbid you");
	}
      else if ( returnVal == 3 ) {
         QMessageBox::about(this,"Bid Results","Bid failed:\nBid amount under the current asking price.");
	}
      else if ( returnVal == 4 ){
         QMessageBox::about(this,"Bid Results","Bid failed:\nProblem with bid quantity");
	}
      else if ( returnVal == 5 ){
         QMessageBox::about(this,"Bid Results","Bid failed:\nAuction has ended.");
	}
      else if ( returnVal == 6 ){
         QMessageBox::about(this,"Bid Results","Bid failed:\nInvalid username or password");
	}
      else 
	{
         QMessageBox::about(this,"Bid Results","Bid failed:\nUnknown reason.");
      cancelBidButton->setEnabled(TRUE);
      bidButton->setEnabled(TRUE);
      snipeButton->setEnabled(TRUE);
      priceLabel->setText("");
      DETAIL_TIMER->stop();
      hide();
      return;
	}
   }
   DETAIL_TIMER->stop();
   hide();
	}

	void BidWindow::CmSnipe()
	{
   doSnipe = TRUE;
   CmBidNow();
	}

	void BidWindow::timerUp()
	{
   char temp[20];
   char temp1[40];
   strcpy(temp1, "Time Left : ");
   stringTimeLeft(theAuction.EndsValue, ebayOffset, temp);
   strcat(temp1, temp);
   timeLabel->setText(temp1);
	}

///////////////////////////////////////////////////////////////////////////////
//////////////////---------QGetNumber Class------------////////////////////////
///////////////////////////////////////////////////////////////////////////////
QGetNumber::QGetNumber(QWidget *parent, const char *name,bool modal, WFlags f )
      : QDialog(parent, name, modal, f) {
   setFixedSize(200,100);
   setGeometry( 100,100,200,100);

   inputLabel = new QLabel( "Enter auction number:", this );
   CHECK_PTR( inputLabel );
   inputLabel->setGeometry(10, 10, 150, 10);
   
   inputEdit = new QLineEdit( this );
   CHECK_PTR( inputEdit );
   inputEdit->setGeometry( 10, 25, 180, 22 );
   connect( inputEdit, SIGNAL(returnPressed()), SLOT(CmOK()) );
   
   CancButton = new QPushButton(this);
   CHECK_PTR(CancButton);
   CancButton->setGeometry(50, 70, 45, 20);
   CancButton->setText("scrap");
   connect( CancButton, SIGNAL(clicked()), SLOT(CmCancel()) );
   
   OKButton = new QPushButton(this);
   CHECK_PTR(OKButton);
   OKButton->setGeometry(105, 70, 45, 20);
   OKButton->setText("save");
   connect( OKButton, SIGNAL(clicked()), SLOT(CmOK()) );
	}
	QGetNumber::~QGetNumber()
	{
	}
	void QGetNumber::CmOK()
	{
   char newName[30];
   strncpy( newName, inputEdit->text(), 30);
   strip_space(newName);
   newName[29] = '\0';
   inputEdit->clear();
   getNumOpen = FALSE;
   hide();
   emit submit_number(newName);
	}
	void QGetNumber::CmCancel()
	{
   inputEdit->clear();
   getNumOpen = FALSE;
   hide();
	}
	void QGetNumber::closeEvent( QCloseEvent * g )
	{
   inputEdit->clear();
   getNumOpen = FALSE;
   hide();
	}
///////////////////////////////////////////////////////////////////////////////
//////////////////---------ConfigMenu Class------------////////////////////////
///////////////////////////////////////////////////////////////////////////////
ConfigMenu::ConfigMenu(QWidget *parent, const char *name,bool modal, WFlags f )
      : QDialog(parent, name, modal, f)
{
   setFixedSize(335,365);
   setGeometry( 100,100,335,365);
   
   userLabel = new QLabel( "Enter user ID:", this );
   CHECK_PTR( userLabel );
   userLabel->setGeometry(10, 10, 150, 10);
   
   userEdit = new QLineEdit( this );
   CHECK_PTR( userEdit );
   userEdit->setGeometry( 10, 25, 280, 22 );
   
   passLabel = new QLabel( "Enter password:", this );
   CHECK_PTR( passLabel);
   passLabel->setGeometry(10, 55, 150, 10);
   
   passEdit = new QLineEdit( this );
   passEdit->setEchoMode(QLineEdit::Password);
   CHECK_PTR( passEdit );
   passEdit->setGeometry( 10, 70, 280, 22 );
   
   browserLabel = new QLabel( "Path to web browser:", this);
   CHECK_PTR( browserLabel );
   browserLabel->setGeometry(10, 100, 150, 10);
   
   browserEdit = new QLineEdit( this );
   CHECK_PTR( browserEdit );
   browserEdit->setGeometry(10, 115, 280, 22);

   setebaypLabel = new QLabel("Set Normal Ebay String",this);
   CHECK_PTR( setebaypLabel );
   setebaypLabel->setGeometry(10,140,280,22);

   setebaypEdit = new QLineEdit(this);
   CHECK_PTR( setebaypEdit );
   setebaypEdit->setGeometry(10,160,280,22);

   listCheck = new QCheckBox( "Track my eBay listings", this );
   CHECK_PTR( listCheck );
   listCheck->setGeometry(10, 210, 150, 15 );
   
   bidCheck = new QCheckBox( "Track current bids", this );
   CHECK_PTR( bidCheck );
   bidCheck->setGeometry(10, 230, 150, 15 );
   
   startCheck = new QCheckBox( "Check auctions on startup", this );
   CHECK_PTR( startCheck );
   startCheck->setGeometry(10, 250, 180, 15);

   deleteCheck = new QCheckBox( "Automatically delete ended auctions", this );
   CHECK_PTR( deleteCheck );
   deleteCheck->setGeometry(10, 270, 250, 15);

   earlyCheck = new QCheckBox( "Remove Auctions that have Ended Early", this );
   CHECK_PTR( earlyCheck );
   earlyCheck->setGeometry(10, 290, 250, 15);

   timeSpinner = new QSpinBox( 1, 600, 1, this);
   CHECK_PTR( timeSpinner );
   timeSpinner->setGeometry(10, 315, 45, 17);

   timeLabel = new QLabel("Snipe timer",this);
   CHECK_PTR( timeLabel );
   timeLabel->setGeometry(62,315,100,13);
   
   fontSpinner = new QSpinBox( 6, 24, 1, this);
   CHECK_PTR( fontSpinner );
   fontSpinner->setGeometry(10, 335, 45, 17);

   fontLabel = new QLabel("Font Size",this);
   CHECK_PTR( fontLabel );
   fontLabel->setGeometry(62,335,100,13);

   boldCheck = new QCheckBox( "Bold", this);
   CHECK_PTR( boldCheck );
   boldCheck->setGeometry(130, 335, 45, 17);
  
   fixedCheck = new QCheckBox( "Fixed Fonts", this);
   CHECK_PTR( fixedCheck ); 
   fixedCheck->setGeometry(130, 315, 75, 17); 

   scrapButton = new QPushButton(this);
   CHECK_PTR(scrapButton);
   scrapButton->setGeometry(175, 335, 45, 20);
   scrapButton->setText("scrap");
   connect( scrapButton, SIGNAL(clicked()), SLOT(CmScrap()) );
   
   keepButton = new QPushButton(this);
   CHECK_PTR(keepButton);
   keepButton->setGeometry(230, 335, 45, 20);
   keepButton->setText("save");
   connect( keepButton, SIGNAL(clicked()), SLOT(CmKeep()) );
	}
	ConfigMenu::~ConfigMenu()
	{
	}
	void ConfigMenu::closeEvent( QCloseEvent * g )
	{
   configOpen = FALSE;
   hide();
	}
	void ConfigMenu::CmKeep()
	{
   char user[76];
   char pass[76];
   char browserpath[200];
   char ebaypath[65];
   bool a,b,c,d,e,ee,ff,gg;
   strcpy(user, userEdit->text());
   encode_password( pass, passEdit->text() );
   strcpy(browserpath, browserEdit->text() );
   strcpy(ebaypath, setebaypEdit->text() );
   a = bidCheck->isChecked();
   b = listCheck->isChecked();
   c = 0;
   d = startCheck->isChecked();
   e = deleteCheck->isChecked();
   ee = earlyCheck->isChecked();
   ff = boldCheck->isChecked();
   gg = fixedCheck->isChecked();
   emit its_done(user, pass, browserpath, b, a, c, d, e, timeSpinner->value(), ebaypath, ee, ff, gg, fontSpinner->value());
   configOpen = FALSE;
   hide();
	}
	void ConfigMenu::CmScrap()
	{
   configOpen = FALSE;
   hide();
	}
	void ConfigMenu::LoadValues(char * name, char * pass, char * browserpath, 
	 bool listings, bool bids, bool alarms, bool startup, 
	bool autoDel, int snipeDelay, char * ebaypath, bool earlycheck, 
	bool boldcheck, bool fixedcheck, int fontSize)
	{
    deleteCheck->setChecked(autoDel);
    earlyCheck->setChecked(earlycheck);
    boldCheck->setChecked(boldcheck);
    fixedCheck->setChecked(fixedcheck);
    startCheck->setChecked(startup);
    timeSpinner->setValue(snipeDelay);
    listCheck->setChecked(listings);
    bidCheck->setChecked(bids);
    passEdit->setText(pass);
    userEdit->setText(name); 
    browserEdit->setText(browserpath);
    setebaypEdit->setText(ebaypath);
    fontSpinner->setValue(fontSize);

	}
/////////////////////////////////////////////////////////////////////
////////////----------DetailsWindow Class---------///////////////////
/////////////////////////////////////////////////////////////////////
DetailsWindow::DetailsWindow( QWidget * parent=0, const char * name=0, bool modal=FALSE,
                 WFlags f=0 )
        : QDialog(parent, name, modal, f){
   setFixedSize(390,310);
   setGeometry(100,100,390,310);

   QColorGroup redColorGroup(red,white,white,white,white,red,white);
   QColorGroup grayColorGroup(white,darkGray,darkGray,darkGray,darkGray,white,darkGray);
   QColorGroup blackColorGroup(white,black,black,black,black,white,black);
   grayPalette = QPalette(grayColorGroup,grayColorGroup,grayColorGroup);
   blackPalette = QPalette(blackColorGroup,blackColorGroup,blackColorGroup);
   redPalette = QPalette(redColorGroup,redColorGroup,redColorGroup);

   DETAIL_TIMER = new QTimer( this );
   CHECK_PTR( DETAIL_TIMER );
   connect( DETAIL_TIMER, SIGNAL(timeout()), SLOT(timerUp()) );

   itemLabel = new QLabel( "", this );
   CHECK_PTR( itemLabel );
   itemLabel->setGeometry(5,5,363,22);
   itemLabel->setAlignment(AlignHCenter | AlignVCenter);
   itemLabel->setPalette(grayPalette);
   itemLabel->setFont(QFont("helvetica", 17, QFont::Bold));

   descriptionLabel = new QLabel( "", this );
   CHECK_PTR( descriptionLabel );
   descriptionLabel->setGeometry(5,27,363,18);
   descriptionLabel->setAlignment(AlignHCenter | AlignVCenter);
   descriptionLabel->setPalette(grayPalette);
   descriptionLabel->setFont(QFont("helvetica", 12, QFont::Normal));
   
   sideBarLabel = new QLabel( "", this );
   CHECK_PTR( sideBarLabel );
   sideBarLabel->setGeometry(5,45,16,247);
   sideBarLabel->setPalette(grayPalette);

   priceLabel = new QLabel( "", this );
   CHECK_PTR( priceLabel );
   priceLabel->setGeometry(21,45,347,20);
   priceLabel->setAlignment(AlignHCenter | AlignVCenter);
   priceLabel->setPalette(redPalette);
   priceLabel->setFont(QFont("helvetica", 17, QFont::Bold));
      
   categoryLabel = new QLabel( "", this );
   CHECK_PTR( categoryLabel );
   categoryLabel->setGeometry(26,70,120,232);
   categoryLabel->setAlignment(AlignRight);
   categoryLabel->setPalette(blackPalette);

   infoLabel = new QLabel( "", this );
   CHECK_PTR( infoLabel );
   infoLabel->setGeometry(146,70,232,232);
   infoLabel->setAlignment(AlignLeft);
   infoLabel->setPalette(blackPalette);

   closeButton = new QPushButton(this);
   CHECK_PTR(closeButton);
   closeButton->setGeometry(152, 275, 90, 20);
   closeButton->setText("close");
   connect( closeButton, SIGNAL(clicked()), SLOT(CmClose()) );
	}
	void DetailsWindow::updateStats()
	{ 
   char info[600];
   char temp[20];
   char temp1[20];
   char temp2[20];
   char unitstring[15];
   char currency[10];
   int infoState = 1;
   // we have 3 states of information:
   // 1 - from aucFile
   // 2 - from ebay user/listings list
   // 3 - full information.

   strip_commas(theAuction.FirstBid);
   if ( theAuction.CurrentBid[0] != 0 ) {
      if ( theAuction.TimeLeft[0] != 0 ) infoState = 3;
      else infoState = 2;
   }   
   strcpy(unitstring, " unit at ");	// DBS
   if( atoi(theAuction.snipeQty) >= 2)	// DBS
       strcpy(unitstring, " units at ");	// DBS
   if ( theAuction.BidCount[0] == '0' && temp[0] != '$') {
       strcpy(theAuction.FirstBid, theAuction.CurrentBid);
     }
      else
     {
     strcpy(temp, theAuction.FirstBid);
     strcpy(temp1, theAuction.CurrentBid);
	}

  if ( theAuction.BidCount[0] != '0') calculateBidIncrement(theAuction.CurrentBid, temp1, currency);
  if ( theAuction.BidCount[0] == '0' ) strcpy(temp1, theAuction.CurrentBid);
     if ( infoState == 3 )
     {

      strcpy(info,"\n ");
      strcat(info,theAuction.Seller);
      strcat(info,theAuction.SellerRate);
      strcat(info,"\n ");
      strcat(info,theAuction.Location);
      strcat(info,"\n ");
      strcat(info,theAuction.Ends);
      strcat(info,"\n ");
      stringTimeLeft(theAuction.EndsValue, ebayOffset, temp2);
      strcat(info,temp2);
      strcat(info,"\n ");
      strcat(info,theAuction.Quantity);
      strcat(info,"\n ");
      strcat(info,theAuction.BidCount);
      strcat(info,"\n ");
      strcat(info,theAuction.HighBidder);
      strcat(info,theAuction.BidderRate);
      strcat(info,"\n ");
      if ( theAuction.BidCount[0] != '0' ) strcat(info,theAuction.currency);
      strcat(info,temp1); 
      strcat(info,"\n ");
      sprintf(temp,theAuction.FirstBid);
      strcat(info,temp);
      strcat(info,"\n ");
      strcat(info,theAuction.BuyitNowPrice);
      strcat(info,"\n ");
      // deal with snipes and quantity	- // DBS
      if ( theAuction.isSnipe && strcmp(theAuction.HighBidder, "Dutch Auction") == 0) {
	  strcat(info, theAuction.snipeQty);
	  strcat(info, unitstring);
	  strcat(info, "$");
	  strcat(info,theAuction.snipeAmount);
	  strcat(info, " each\n ");
      }
      else
      {
	  if( theAuction.isSnipe )
	  {
	      strcat(info,"$");
	      strcat(info,theAuction.snipeAmount);
	  }
	  strcat(info,"\n ");
      }
      strcat(info,theAuction.snipeKey);
   }
   else
   {
      if ( theAuction.isSnipe ) {
         strcpy(info, "\n\n\n\n\n\n\n\n\n\n ");
         strcat(info, theAuction.snipeQty);	// DBS
         strcat(info, unitstring);		// DBS
         strcat(info, theAuction.snipeAmount);
         strcat(info, " each\n ");		// DBS
         strcat(info, theAuction.snipeKey);
      }
      else strcpy(info,"");
   }
   infoLabel->setText(info);
	}
	void DetailsWindow::fillBlanks(struct BidMakeup_type aa, char * ID, char * PASS, int offSet)
	{ 
   ebayOffset = offSet;
   strcpy(tempID, ID);
   strcpy(tempPASS, PASS);
   theAuction = aa;
   categoryLabel->setText("\nSeller :\n"
                         "Location :\n"
                         "Ends :\n"
                         "Time Left :\n"
                         "Quantity :\n"
                         "Number of Bids :\n"
                         "High Bidder :\n"
                         "Minimum Bid :\n"
                         "First Bid :\n"
			 "Buy it Now:\n"
                         "Snipe Amount :\n"
                         "Snipe Key :\n");

   char info[600];
   int infoState = 1;
   // we have 3 states of information:
   // 1 - from aucFile
   // 2 - from ebay user/listings list
   // 3 - full information.
   if ( aa.CurrentBid[0] != 0 ) {
      if ( aa.TimeLeft[0] != 0 ) infoState = 3;
      else infoState = 2;
   }   
   strcpy(info, "Item Number : ");
   strcat(info, theAuction.ItemNumber);
   itemLabel->setText(info);
   strcpy(info, "Current Bid : ");
   if ( infoState > 1 ) {
      strcat(info, theAuction.CurrentBid);
      if (infoState == 3) {
         if ( aa.reserveMet == 'y' ) strcat(info, " - reserve met");
         else if ( aa.reserveMet == 'n' ) strcat(info, " - reserve not met");
         else if ( aa.reserveMet == 'x' ) strcat(info, " - no reserve");
      }
   }
   priceLabel->setText(info);
   descriptionLabel->setText(theAuction.Description);

   updateStats();

   DETAIL_TIMER->start(1000,FALSE);      
	}
	void DetailsWindow::timerUp()
	{
   updateStats();
	}
   
	void DetailsWindow::CmClose()
	{
   DETAIL_TIMER->stop();   
   hide();
	}
/////////////////////////////////////////////////////////////////////
////////////----------QFileView Class---------///////////////////
/////////////////////////////////////////////////////////////////////
QFileView::QFileView( QWidget * parent=0, const char * name=0, bool modal=FALSE,
                 WFlags f=0 )
        : QDialog(parent, name, modal, f){
   setFixedSize(500,300);
   setGeometry(100,100,500,300);

   topLabel = new QLabel( "", this );
   CHECK_PTR( topLabel );
   QFont f("Helvetica", 14, QFont::Bold);
   topLabel->setFont( f );
   topLabel->setAlignment( AlignCenter );
   topLabel->setGeometry(5,5,490,20);

   viewArea = new QMultiLineEdit(this);
   CHECK_PTR( viewArea );
   viewArea->setGeometry(5,30,490,240);
   viewArea->setReadOnly(TRUE);
      
   closeButton = new QPushButton(this);
   CHECK_PTR(closeButton);
   closeButton->setGeometry(295, 275, 50, 20);
   closeButton->setText("close");
   connect( closeButton, SIGNAL(clicked()), SLOT(CmClose()) );
   
   monthButton = new QPushButton(this);
   CHECK_PTR(monthButton);
   monthButton->setGeometry(5, 275, 160, 20);
   monthButton->setText("view last month");
   connect( monthButton, SIGNAL(clicked()), SLOT(CmChangeMonth()) );   
	}
	void QFileView::fillBlanks(char * fileToView, bool isItThisMonth)
	{    
   thisMonth = isItThisMonth;
   if ( thisMonth )
   {
      topLabel->setText("This month's snipes");
      monthButton->setText("view last month");
      QFile theFile( fileToView );
      if ( !theFile.open( IO_ReadOnly ) ) {
         viewArea->clear();
         topLabel->setText("No snipes executed this month");
         return;
      }
      viewArea->clear();
      QTextStream t(&theFile);
      while ( !t.eof() ) {
         QString s = t.readLine();
         viewArea->append( s );
      }
      theFile.close();
   }
   else
   {
      topLabel->setText("Last month's snipes");
      monthButton->setText("view this month");
      QFile theFile( fileToView );
      if ( !theFile.open( IO_ReadOnly ) ) {
         viewArea->clear();
         topLabel->setText("No snipes executed last month");
         return;
      }
      viewArea->clear();
      QTextStream t(&theFile);
      while ( !t.eof() ) {
         QString s = t.readLine();
         viewArea->append( s );
      }
      theFile.close();
   }
	}
	void QFileView::CmClose()
	{
   hide();
	}
	void QFileView::CmChangeMonth()
	{
   if ( thisMonth ) thisMonth = FALSE;
   else thisMonth = TRUE;
   char * homedir = getenv("HOME");
   char fileName[200];
   MakeFileName(homedir, "/.bidwatcher/snipe", fileName, thisMonth);
   fillBlanks(fileName, thisMonth);
	}
/////////////////////////////////////////////////////////////////////
////////////----------QEndedView Class---------///////////////////
/////////////////////////////////////////////////////////////////////
QEndedView::QEndedView( QWidget * parent=0, const char * name=0, bool modal=FALSE,
                 WFlags f=0 )
        : QDialog(parent, name, modal, f){
   setFixedSize(500,300);
   setGeometry(100,100,500,300);

   topLabel = new QLabel( "", this );
   CHECK_PTR( topLabel );
   QFont f("Helvetica", 14, QFont::Bold);
   topLabel->setFont( f );
   topLabel->setAlignment( AlignCenter );
   topLabel->setGeometry(5,5,490,20);

   viewArea = new QMultiLineEdit(this);
   CHECK_PTR( viewArea );
   viewArea->setGeometry(5,30,490,240);
   viewArea->setReadOnly(TRUE);
      
   closeButton = new QPushButton(this);
   CHECK_PTR(closeButton);
   closeButton->setGeometry(295, 275, 50, 20);
   closeButton->setText("close");
   connect( closeButton, SIGNAL(clicked()), SLOT(CmClose()) );
   
   monthButton = new QPushButton(this);
   CHECK_PTR(monthButton);
   monthButton->setGeometry(5, 275, 160, 20);
   monthButton->setText("view last month");
   connect( monthButton, SIGNAL(clicked()), SLOT(CmChangeMonth()) );   
	}
	void QEndedView::fillBlanks(char * fileToView, bool isItThisMonth)
	{    
   thisMonth = isItThisMonth;
   if ( thisMonth )
   {
      topLabel->setText("This month's ended auctions");
      monthButton->setText("view last month");
      QFile theFile( fileToView );
      if ( !theFile.open( IO_ReadOnly ) ) {
         viewArea->clear();
         topLabel->setText("No deleted auctions this month");
         return;
      }
      viewArea->clear();
      QTextStream t(&theFile);
      while ( !t.eof() ) {
         QString s = t.readLine();
         viewArea->append( s );
      }
      theFile.close();    
   }
   else
   {
      topLabel->setText("Last month's ended auctions");
      monthButton->setText("view this month");
      QFile theFile( fileToView );
      if ( !theFile.open( IO_ReadOnly ) ) {
         viewArea->clear();
         topLabel->setText("No deleted auctions last month");
         return;
      }
      viewArea->clear();
      QTextStream t(&theFile);
      while ( !t.eof() ) {
         QString s = t.readLine();
         viewArea->append( s );
      }
      theFile.close();
   }
	}
	void QEndedView::CmClose()
	{
   hide();
	}
	void QEndedView::CmChangeMonth()
	{
   if ( thisMonth ) thisMonth = FALSE;
   else thisMonth = TRUE;
   char * homedir = getenv("HOME");
   char fileName[200];
   MakeFileName(homedir, "/.bidwatcher/log", fileName, thisMonth);
   fillBlanks(fileName, thisMonth);
	}
///////////////////////////////////////////////////////////////////////////////
////////////////////-------------bidwatcher-------------/////////////////////
///////////////////////////////////////////////////////////////////////////////
BidWatcher::BidWatcher( QWidget *parent, const char *name )
    : QWidget( parent, name ){
   configOpen = getNumOpen = cancelPressed = clockIsSet = FALSE;
   updateInProgress = FALSE;
   trackBids = trackListings = autoDelete = doStartup = earlycheck = TRUE;
   bidIdx = aucIdx = Timer1Idx = bidIdx = timeDiff = 0;   
   snipeDelay = 15;
   setSizeIncrement( 1, 13);
   authID[0] = 0; authPASS[0] = 0;
   strcpy(browserPATH, "netscape");
   strcpy(ebayPATH, "/ws/eBayISAPI.dll");
   
   umask( 0077 );		// make sure files are private

   char fileName[200];
   char * homeDir = getenv("HOME");
   strcpy(fileName, homeDir);
   strcat(fileName, "/.bidwatcher");
   mkdir(fileName, 0700);


   ReadaucFile();

   int ms=aucIdx*17;
   int ms2=ms*2;
   if ( ms < 210 ) ms=210;
   if ( ms2 < 750 ) ms2=750;
   if ( boldcheck && !fixedcheck) {
	ms=ms+45; 
	ms2=ms2+130;
	}
   if ( fixedcheck ) {
	ms=ms+55; 
	ms2=ms2-210;
	}
   if ( ms > 560 ) ms=570;
   if ( ms2 > 950 ) ms2=1070;
   if ( ms2 < 950 ) ms2=1070;
   setGeometry( 150,90,ms2-140,ms+5);

   if ( boldcheck && !fixedcheck) setGeometry( 150,90,ms2-160,ms-40);
   if ( fixedcheck ) setGeometry( 150,90,ms2-40,ms);
   if ( boldcheck && fixedcheck) setGeometry( 150,90,ms2-170,ms-20);

   QApplication::setWinStyleHighlightColor(QColor(150, 150, 150));
   QColor fg(0,255,0);
   QColor bg(0,0,0);
   QColor fg1(255,255,0);
   QColor fg2(255,0,0);

   QColorGroup grayColorGroup(darkGray,lightGray,white,gray,gray,bg3,bg2);
   grayPalette = QPalette(grayColorGroup,grayColorGroup,grayColorGroup);
   QColorGroup yellowColorGroup(fg1,bg,fg1,bg,bg,fg1,bg);
   QColorGroup greenColorGroup(fg,bg,fg,bg,bg,fg,bg);
   QColorGroup redColorGroup(fg2,bg,fg2,bg,bg,fg2,bg);
   yellowPalette = QPalette(yellowColorGroup,yellowColorGroup,yellowColorGroup);
   greenPalette = QPalette(greenColorGroup,greenColorGroup,greenColorGroup);
   redPalette = QPalette(redColorGroup,redColorGroup,redColorGroup);
   
   TIMER1 = new QTimer( this );
   CHECK_PTR( TIMER1 );
   connect( TIMER1, SIGNAL(timeout()), SLOT(timer1Up()) );
   
   TIMER2 = new QTimer ( this );
   CHECK_PTR( TIMER2 );
   connect( TIMER2, SIGNAL(timeout()), SLOT(timer2Up()) );
   
   TIMER3 = new QTimer( this );
   CHECK_PTR( TIMER3 );
   connect( TIMER3, SIGNAL(timeout()), SLOT(timer3Up()) );
   
   TIMER4 = new QTimer( this );
   CHECK_PTR(TIMER4);
   connect(TIMER4, SIGNAL(timeout()), SLOT(timer4Up()));

   aucList = new NewListBox( this );
   CHECK_PTR( aucList );


   QFont f("clean", fontSize, QFont::Normal);
   if ( boldcheck )  f = QFont("clean", fontSize, QFont::Bold);
   if ( fixedcheck )  f = QFont("fixed", fontSize, QFont::Normal);
   if ( fixedcheck && boldcheck )  f = QFont("fixed", fontSize, QFont::Bold);
   aucList->setFont(f);
   aucList->setGeometry( 0, 44, width()-1, height()-44 );
   aucList->setPalette(grayPalette);
   aucList->setAutoScrollBar(TRUE);
   aucList->setBottomScrollBar(FALSE);
   aucList->setAutoBottomScrollBar(FALSE);
   aucList->setSmoothScrolling(TRUE);
   aucList->setFocusPolicy(QWidget::NoFocus);
   connect( aucList, SIGNAL( selected(int) ), SLOT( doubleClick(int) ) );
   connect( aucList, SIGNAL( RightClicked()), SLOT( CmMenu() ) );
   connect( aucList, SIGNAL( MiddleClicked()), SLOT( CmMenu() ) );

   columnHeaderLabel = new QLabel(this);
   CHECK_PTR(columnHeaderLabel);
   columnHeaderLabel->setGeometry(14,30,700,14);
   columnHeaderLabel->setFont(f);
   columnHeaderLabel->setText("   Number   | Current     |  Time Left     | BuY it/Snipe      |   Description ");
   listSelect = new QPopupMenu;
   CHECK_PTR( listSelect );
   listSelect->insertItem( "show item details", this, SLOT(CmDetails()) );
   listSelect->insertItem( "open browser to this auction", this, SLOT(CmWeb() ) );
   listSelect->insertItem( "update this auction", this, SLOT(UpdateItem()) );
   listSelect->insertItem( "bid / set up snipe", this, SLOT(CmBid() ) );
   listSelect->insertItem( "cancel snipe", this, SLOT(CmUnBid() ) );
   listSelect->insertItem( "delete this item", this, SLOT(deleteItem() ) );
   listSelect->insertItem( "delete ALL auctions", this, SLOT(flush() ) );
   listSelect->insertItem( "view completed snipes", this, SLOT(CmViewFile()));
   listSelect->insertItem( "view completed auctions", this, SLOT(CmViewEnded()));

   
   brightLabel = new QLabel ( this );
   CHECK_PTR( brightLabel );
   brightLabel->setGeometry( 467,5,14,14  );
   brightLed.loadFromData(light_data, light_len);
   dimLed.loadFromData(dark_data, dark_len);
   brightLabel->setPixmap(dimLed);
   
   statLabel = new QLabel(this);
   CHECK_PTR(statLabel);
   statLabel->setGeometry(275, 5, 140, 14);
   statLabel->setPalette(greenPalette);
   f = QFont("clean", 12);
   statLabel->setFont(f);
   
   timeLabel = new QLabel(this);
   CHECK_PTR(timeLabel);
   timeLabel->setGeometry(412,5,55,14);
   timeLabel->setPalette(greenPalette);

   maskDot.loadFromData(mask_data, mask_len);
   maskSnipe.loadFromData(snipemask_data, snipemask_len);
   redDot.loadFromData(red_data, red_len);
   redDot.setMask(maskDot);
   blueDot.loadFromData(blue_data, blue_len);
   blueDot.setMask(maskDot);
   blackDot.loadFromData(black_data, black_len);
   blackDot.setMask(maskDot);
   greenDot.loadFromData(green_data, green_len);
   greenDot.setMask(maskDot);
   snipeDot.loadFromData(snipe_data, snipe_len);
   snipeDot.setMask(maskSnipe);
   
   //////////////////////////// DEFINE BUTTONS /////////////////////
   
   UserButton = new QToolButton( this );
   CHECK_PTR( UserButton );
   UserButton->setGeometry( 4,1,31,22 );
   id_btn.loadFromData(user_data, user_len);
   UserButton->setUsesBigPixmap(TRUE);
   UserButton->setPixmap(id_btn);
   connect(UserButton, SIGNAL( clicked() ), SLOT( CmUser() ) );
   QToolTip::add( UserButton, "preferences");
   
   AddButton = new QToolButton( this );
   CHECK_PTR( AddButton );
   AddButton->setGeometry( 38,1,31,22 );
   add_btn.loadFromData(plus_data, plus_len);
   AddButton->setUsesBigPixmap(TRUE);
   AddButton->setPixmap(add_btn);
   connect(AddButton, SIGNAL( clicked() ), SLOT( CmAddItem() ) );
   QToolTip::add( AddButton, "add auction by number");
   
   ClockButton = new QToolButton( this );
   CHECK_PTR( ClockButton );
   ClockButton->setGeometry( 69,1,31,22 );
   clock_btn.loadFromData(clock_data,clock_len);
   ClockButton->setUsesBigPixmap(TRUE);
   ClockButton->setPixmap(clock_btn);
   connect(ClockButton, SIGNAL( clicked() ), SLOT( CmTimeSync() ) );
   QToolTip::add( ClockButton, "sync clock");
   
   UpdateButton = new QToolButton( this );
   CHECK_PTR( UpdateButton );
   UpdateButton->setGeometry( 100,1,31,22 );
   update_btn.loadFromData(update_data, update_len);
   UpdateButton->setUsesBigPixmap(TRUE);
   UpdateButton->setPixmap(update_btn);
   connect(UpdateButton, SIGNAL( clicked() ), SLOT( CmUpdate() ) );
   QToolTip::add( UpdateButton, "pull info from ebay NOW");
   
   HelpButton = new QToolButton( this );
   CHECK_PTR( HelpButton );
   HelpButton->setGeometry( 131,1,31,22 );
   help_btn.loadFromData(help_data, help_len);
   HelpButton->setUsesBigPixmap(TRUE);
   HelpButton->setPixmap(help_btn);
   connect(HelpButton, SIGNAL( clicked() ), SLOT( CmHelp() ) );
   QToolTip::add( HelpButton, "about bidwatcher");
   
   ExitButton = new QToolButton( this );
   CHECK_PTR( ExitButton );
   ExitButton->setGeometry( 162,1,31,22 );
   exit_btn.loadFromData(exit_data, exit_len);
   ExitButton->setUsesBigPixmap(TRUE);
   ExitButton->setPixmap(exit_btn);
   connect(ExitButton, SIGNAL( clicked() ), SLOT( CmExit() ) );
   QToolTip::add( ExitButton, "quit");
   
   cancelButton = new QToolButton( this );
   CHECK_PTR( cancelButton );
   cancelButton->setGeometry( 193, 1, 31, 22);
   canc_btn.loadFromData(stop_data, stop_len);
   cancelButton->setUsesBigPixmap(TRUE);
   cancelButton->setPixmap(canc_btn);
   connect(cancelButton, SIGNAL( clicked() ), SLOT( CmCancelClicked() ) );
   cancelButton->hide();

   if ( strlen(authID) > 1 )
   {
      char capt[150];
      strcpy(capt, "qtbidwatcher - ");
      strcat(capt, authID);
      setCaption( capt );
   }
   else setCaption("qtbidwatcher");
   firstMenu = new ConfigMenu(0,"config", FALSE, WStyle_Customize | WStyle_DialogBorder);
   connect(firstMenu, SIGNAL( its_done(char *, char *,char *, bool, bool, bool, bool, bool, int, char *, bool, bool, bool, int) ), SLOT( changeName(char *, char *, char *, bool, bool, bool, bool, bool, int, char *, bool, bool, bool, int) ) );
   getNumWin = new QGetNumber(0, "getNum", FALSE, 0);
   connect(getNumWin, SIGNAL( submit_number(char *) ), SLOT ( addNewItem(char *) ) ); 
   bidWindow = new BidWindow();
   connect(bidWindow, SIGNAL( setsnipe(BidMakeup_type &,int)), SLOT ( setupSnipe(BidMakeup_type &, int) ));
   detailsWindow = new DetailsWindow();
   viewFileWindow = new QFileView();
   viewEndedWindow = new QEndedView();
   UpdateList();
   TIMER1->start(100,TRUE);          // runs one time at startup
   TIMER2->start(1000,FALSE);        // controls the clock update / snipe execution
   TIMER3->start(BIGDELAY, TRUE);    // long update
   TIMER4->start(30*1000, TRUE);     // until we know better, don't waste bandwidth
	} // end constructor

BidWatcher::~BidWatcher()
	{
   WriteAucFile();
	}
/////////////////////////////////////////////////////////////////////////////
// WriteAucFile
//   writes the configuration file
/////////////////////////////////////////////////////////////////////////////
void BidWatcher::WriteAucFile() {
   ofstream outFile;
   int rows = 5;
   char fileName[200];
   char * homeDir = getenv("HOME");
   strcpy(fileName, homeDir);
   strcat(fileName, "/.bidwatcher/bw.cfg");
   outFile.open(fileName);
   if ( outFile )
   {
      outFile << "1 " << rows << " " << soundOn << " " << trackListings << " " << trackBids << " " << autoDelete << " " << doStartup <<
      " " << snipeDelay << " " << clockIsSet << " " << timeDiff << " " << earlycheck << " " << boldcheck << " " << fixedcheck << " " << fontSize << endl;
      outFile << "2 " << authID << endl;
      outFile << "3 " << authPASS << endl;
      outFile << "4 " << browserPATH << endl;
      outFile << "10 " << ebayPATH << endl;
      for ( int i=0; i < aucIdx; i++ )
      {
         outFile << "5 " << BidMakeup[i].ItemNumber << "\t" << BidMakeup[i].Description << endl;
         outFile << "6 " << BidMakeup[i].stat << " " <<  BidMakeup[i].EndsValue 
         << " " << BidMakeup[i].isSnipe << endl;;
         if ( BidMakeup[i].isSnipe )
         {
            outFile << "7 " << BidMakeup[i].snipeQty << "\t" << BidMakeup[i].snipeAmount << "\t";
            outFile << BidMakeup[i].snipeKey << endl;
         }
      }
   }
   outFile.close();
	}
/////////////////////////////////////////////////////////////////////////////
// User:
//      User presses the user button - puts the line edit box on the
//      screen and a cancel button.
// Calls:
//      none 
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::CmUser(){
   if ( configOpen )
       return;
   else
   {
       char authPASS_cleaned[ 76 ];
       decode_password( authPASS_cleaned, authPASS );

       firstMenu->LoadValues(authID, authPASS_cleaned, browserPATH, trackListings, trackBids, soundOn, doStartup, autoDelete, snipeDelay ,ebayPATH, earlycheck, boldcheck, fixedcheck, fontSize);
       configOpen = TRUE;
       firstMenu->show();
   }
	}
//////////////////////////////////////////////////////////////////////////////
// CmAddItem:
//      User presses the add button, puts a line edit box in the uppder
//      right corner and a cancel button.
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::CmAddItem(){
   if ( getNumOpen ) return;
   else
   {
      getNumOpen = TRUE;
      getNumWin->show();
   }
	}
//////////////////////////////////////////////////////////////////////////////
// CmTimeSync
////////////////////////////////////////////////////////////////////
void BidWatcher::CmTimeSync(){
   if ( updateInProgress ) return;
   updateInProgress = TRUE;
   TimeSync();
   updateInProgress = FALSE;
	}
///////////////////////////////////////////////////////////////////////////////
// CmUpdate
//        responds to clicking the update button, 
//        checks for new auctions and updates the whole list of 
//        auctions.
///////////////////////////////////////////////////////////////////////////////
void BidWatcher::CmUpdate(){

   if ( updateInProgress ) return;
   updateInProgress = TRUE;
   int returnVal;

   returnVal = GetUserBids();
   if ( returnVal == 1 )
   {
      GetUserListings();
      UpdateList();
   }
   else if (  returnVal == -2 )
   {
      QMessageBox::about(this,"Error","Invalid username.");
   }
   else
   {
      QMessageBox::about(this,"Error","Network error.");
   }
   if ( aucIdx != 0 )
   {
      UpdateAll(FALSE);
   }
   updateInProgress = FALSE;
	}
//////////////////////////////////////////////////////////////////////////////
// CmHelp:
//       user presses the view button, just grabs the index of the selected
//      item from the QListbox aucList and passes it to DisplayDetais, 
//      which does the real work.
// calls:
//     
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::CmHelp(){
   char helptext[1024];
   strcpy(helptext, "Bidwatcher [ v");
   strcat(helptext, versionInfo);
   strcat(helptext, " ]\n\n"
	  "Authors: \n"
	  "Trent McNair <trent@rmci.net>,\n"
	  "Tom McNair <tmcnair@cyberhighway.net>\n"
	  "\n"
	  "Maintainer:\n"
	  "Thomas Enstall <tht@nichewebworks.com>\n"
	  "\n"
	  "Ebay Motors Patch and Fix for 03-14-2002 Ebay Pages:\n"
	  "Thomas Enstall <tht@nichewebworks.com>\n"
	  "\n"
	  "All copyrights under the GPL 1998-2002\n"
	  "\n"
	  "Send all questions, bug reports, etc: to tht@nichewebworks.com\n"
	  "\n"
	  "For complete documentation, see the web pages that came\n"
	  "with Bidwatcher, or view them at: \n"
	  "http://www.midwestcs.com/bidwatcher/doc/quick_start.html"
       );
   QMessageBox::about(this,"bidwatcher", helptext);
	}
	void BidWatcher::CmDetails()
	{
   struct BidMakeup_type newAuction;
   int choice = aucList->currentItem();
   if ( choice < 0 ) return;
   newAuction = BidMakeup[choice];
   detailsWindow->fillBlanks(newAuction, authID, authPASS, timeDiff);
   detailsWindow->show();   
	}
	void BidWatcher::CmViewFile()
	{
   char * homedir = getenv("HOME");
   char fileName[200];
   MakeFileName(homedir, "/.bidwatcher/snipe", fileName, TRUE);
   viewFileWindow->fillBlanks(fileName, TRUE);
   viewFileWindow->show();
	}
	void BidWatcher::CmViewEnded()
	{
   char * homedir = getenv("HOME");
   char fileName[200];
   MakeFileName(homedir, "/.bidwatcher/log", fileName, TRUE);
   viewEndedWindow->fillBlanks(fileName, TRUE);
   viewEndedWindow->show();
	}
/////////////////////////////////////////////////////////////////////////////////
// CmWeb
//   Launches a web browser session for the selected auction.
/////////////////////////////////////////////////////////////////////////////////
void BidWatcher::CmWeb(){
   int choice = aucList->currentItem();
   launchBrowser( choice );
	}

	void BidWatcher::launchBrowser( int choice )
	{
   char commandLine[500];
   if ( strlen(browserPATH) < 2 )
   {
      QMessageBox::about(this,"bidwatcher", "No Web Browser configured!");
      return;
   }
   char homePath[200];
   //check if netscape is running:
   char lockBuffer[200];
   strcpy(homePath,getenv("HOME"));
   strcat(homePath, "/.netscape/lock");
   int err = readlink(homePath, lockBuffer, 199);
   bool hit = TRUE;
   if ( err < 0 ) hit = FALSE;
   if ( (choice < 0) || (choice > aucIdx) ) return;
   
   else if ( (strstr(browserPATH, "scape") != NULL) && hit )
   {
      strcpy(commandLine, "exec ");
      strcat(commandLine, browserPATH);
      strcat(commandLine, " -remote \"openURL(http://cgi.ebay.com/ws/eBayISAPI.dll?ViewItem&item=");
      strcat(commandLine, BidMakeup[choice].ItemNumber);
      strcat(commandLine, ",new-window)\"");
      system(commandLine);
   }
   else
   {
      strcpy( commandLine, "exec ");
      strcat( commandLine, browserPATH);
      strcat( commandLine, " http://cgi.ebay.com/ws/eBayISAPI.dll?ViewItem\\&item=");
      strcat( commandLine, BidMakeup[choice].ItemNumber);
      strcat( commandLine, " &");
      system(commandLine);
   }
	}
//////////////////////////////////////////////////////////////////////////////
// CmBid
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::CmBid(){
   struct BidMakeup_type newAuction;
   int choice = aucList->currentItem();
   newAuction = BidMakeup[choice];
   if ( strlen(newAuction.TimeLeft) < 3 )
   {
      statLabel->setText(" updating auction...");
      int returnVal = UpdateAuction(newAuction);
      if ( returnVal != 1 )
      {
         QMessageBox::about(this,"Error","Can't update auction, network problems.");
         statLabel->setText("");
         return;
      }
      BidMakeup[choice] = newAuction;
      UpdateListItem(choice);
   }
   statLabel->setText("");
   bidWindow->fillBlanks(BidMakeup[choice], choice, authID, authPASS, timeDiff, currentBid);
   bidWindow->show();     
	}
//////////////////////////////////////////////////////////////////////////
// ClearSnipe
//////////////////////////////////////////////////////////////////////////
void BidWatcher::ClearSnipe(int choice){
   BidMakeup[choice].isSnipe = FALSE;
   strcpy(BidMakeup[choice].snipeKey, "");
   strcpy(BidMakeup[choice].snipeAmount, "");
   strcpy(BidMakeup[choice].snipeQty, "");
   UpdateListItem(choice);
   WriteAucFile();
	}
//////////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////////
void BidWatcher::CmUnBid(){
   int indx = aucList->currentItem();
   ClearSnipe(indx);
	}
//////////////////////////////////////////////////////////////////////////////
// CmExit:
//      triggered by the exit button, this properly shuts down the program.
// calls:
//      none
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::CmExit(){
   if ( updateInProgress == TRUE ) return;
   qApp->quit();
	}
//////////////////////////////////////////////////////////////////////////////
// CmCancelClicked:
//      user presses the cancel button. This changes cancelPressed to
//      to TRUE, which when encountered by the UpdateAll loop, will cancel
//      that operation (which if there is a large number of auctions, can
//      be quite lengthy.  Also clears edit boxes from the window.
// calls:
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::CmCancelClicked(){
   cancelPressed = TRUE;
	}
/////////////////////////////////////////////////////////////////////////////
//  UpdateItem
//      responds to update this auction popup menu selection
/////////////////////////////////////////////////////////////////////////////
void BidWatcher::UpdateItem(){
   int clickIndex = aucList->currentItem();

   if ( updateInProgress == TRUE ) return;
   updateInProgress = TRUE;
   struct BidMakeup_type NewAuction;
   statLabel->setText(" updating auction...");
   statLabel->repaint();
   NewAuction = BidMakeup[clickIndex];
   int returnVal = UpdateAuction(NewAuction);

   if ( returnVal == 3 )
   {
      QMessageBox::about(this,"Error","Invalid auction number.1");
   }
   else if (returnVal == 4)
   {
      QMessageBox::about(this,"Error","Timeout.");
   }
   else if (returnVal == 5)
   {
      QMessageBox::about(this,"Error","CGI server down.");
   }
   else if ( returnVal == 6 ) {
         statLabel->setText(" Removing Invalid Auction...");
         DeleteAuction(clickIndex);
         aucList->removeItem(clickIndex);
         clickIndex--;
        returnVal = 6;
        }

   else if (returnVal  == 2)
   {
      QMessageBox::about(this,"Error","Can't connect to server.");
   }
   else if ( returnVal != 10 )
   {
      BidMakeup[clickIndex] = NewAuction;
      UpdateListItem(clickIndex);
   }

   statLabel->setText("");
   updateInProgress = FALSE;
	}
//////////////////////////////////////////////////////////////////////////////
// deleteItem:
//      removes the selected item 
//      by the hilighted item in the listbox aucList.
// calls:
//      DeleteAuction(), UpdateList()
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::deleteItem(){
   if ( updateInProgress == TRUE ) return;
   updateInProgress = TRUE;
   int choice = aucList->currentItem();
   if ( choice < 0 )
   { 
      updateInProgress = FALSE;
      return;  // no item selected
   }
   DeleteAuction(choice);
   aucList->removeItem(choice);
   updateInProgress = FALSE;
   if ( choice == (aucIdx) )
       aucList->setCurrentItem(choice - 1);
   else
       aucList->setCurrentItem(choice);
   WriteAucFile();
	}
//////////////////////////////////////////////////////////////////////////////
// flush:
//      all bets off, clears all the structs and emptys the .audFile
// calls:
//      DeleteAuction(), UpdateList()
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::flush(){
   int totalAucs = aucIdx;
   int i;

   if ( updateInProgress == TRUE )
       return;

   if( totalAucs <= 0 )
       return;

   updateInProgress = TRUE;

   i = QMessageBox::warning(
       this, "Delete ALL Auctions",
       "Warning:  Are you sure you want to delete ALL auctions?",
       "Yes", "No", NULL, 1, 1 );

   if( i != 0 )
   {
       updateInProgress = FALSE;
       return;
   }

   for (i = 0; i < totalAucs; i++)
   {
      ClearBidMakeUp(BidMakeup[i]);
   }
   aucIdx = 0;
   bidIdx = 0;
   setCaption( "qtbidwatcher" );
   UpdateList();
   updateInProgress = FALSE;
   WriteAucFile();
	}
////////////////////////////////////////////////////////////////////////////
// changeName:
//      submit a change to the recorded user ID.  does some initial
//      error checking on the submitted name, then calls GetUserAuctions
//      to get the users auctions from ebay and verify the ID.
//   calls:
//      GetUserAuctions(), UpdateList(), 
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::changeName(char * name, char * pass, char * path, bool a, bool b, 
bool c, bool d, bool e, int sd, char * ePATH, bool f, bool g, bool h, int fs){
   if ( updateInProgress ) return;
   if ( strcmp(authID, name) != 0 )
   {
   }
   fontSize = fs;
   snipeDelay = sd;
   strcpy(authID, name);
   strcpy(authPASS, pass);
   strcpy(browserPATH, path);
   strcpy(ebayPATH, ePATH);
   trackListings = a;
   trackBids = b;
   soundOn = c;
   doStartup = d;   
   autoDelete = e;
   earlycheck = f;
   boldcheck = g;
   fixedcheck = h;
   WriteAucFile();
   if ( strlen(authID) > 1 )
   {
      char capt[150];
      strcpy(capt, "qtbidwatcher - ");
      strcat(capt, authID);
      setCaption( capt );
   }
   else setCaption("qtbidwatcher");
	} // end changeName
//////////////////////////////////////////////////////////////////////////////
// UpdateLatest
//      updates auctions with less than 4 hours remaining
///////////////////////////////////////////////////////////////////////////////
void BidWatcher::UpdateLatest(){
   cancelButton->show();
   cancelPressed = FALSE;
   struct BidMakeup_type newAuction;
   int returnVal = 0;
   int intTimeLeft;
   int minTimeLeft;
   char numerator[4];
   char denominator[4];
   char message[20];
   int err;
   int prev_selected = aucList->currentItem();

   for ( int i = 0; i < aucIdx; i++ ) 
   {
      if ( cancelPressed == TRUE )
	  break;

      minTimeLeft = timeToNextEnd() - (int)((avg_load_time + 500)/1000);

      if( minTimeLeft > -2 && minTimeLeft < snipeDelay + 2 )	// don't interfere with the snipe
	  break;

      strcpy(message, " updating ");
      err = sprintf(denominator, "%d", aucIdx);
      err = sprintf(numerator, "%d", (i + 1));
      strcat(message, numerator);
      strcat(message, "/");
      strcat(message, denominator);
      statLabel->setText(message);

      qApp->processEvents();
      intTimeLeft = CalcTimeLeft(BidMakeup[i].EndsValue, timeDiff);
      if ( (intTimeLeft < AUTODEL)
	   && (BidMakeup[i].TimeLeft[0] != 0) && (autoDelete) )
      {
         DeleteAuction(i);
         aucList->removeItem(i);
         i--;
      }
      else if ( (BidMakeup[i].TimeLeft[0] == 0)
		|| (intTimeLeft > -60 && intTimeLeft < 4*60*60) )
      {
          newAuction = BidMakeup[i];
          returnVal = UpdateAuction(newAuction);

          if ( returnVal == 1 )
          {
             BidMakeup[i] = newAuction;
             UpdateListItem(i);
          }

#if 0
	  /* wrs 02/06/00 -- I don't want these checks.  transient errors
	   * should not silently cause an auction to be deleted, and you
	   * should continue updating other auctions info... */
          else if ( returnVal == 3 )
          {
             DeleteAuction(i);
             aucList->removeItem(i);
             i--;
          }
	else if ( returnVal == 6 ) {
         statLabel->setText(" Removing Invalid Auction...");
         DeleteAuction(idx);
         aucList->removeItem(i);
         i--;
        returnVal = 6;
        }

          else
          {
	      statLabel->setText("");
	      cancelButton->hide();
	      cancelPressed = FALSE;
	      return;
          }
 #endif
      }
      else 
      {
          UpdateListItem(i);
      }
   }
   aucList->setCurrentItem( prev_selected );
   aucList->centerCurrentItem();
   statLabel->setText("");
   cancelButton->hide();
   cancelPressed = FALSE;
   return;
	}
//////////////////////////////////////////////////////////////////////////////
// timeToNextEnd
//      Calculate the time until the next auction ends
//////////////////////////////////////////////////////////////////////////////
int BidWatcher::timeToNextEnd(){
   int intTimeLeft;
   int minTimeLeft = 9999999;

   for ( int i = 0; i < aucIdx; i++ ) 
   {
      intTimeLeft = CalcTimeLeft(BidMakeup[i].EndsValue, timeDiff);
      if( intTimeLeft > -30 && intTimeLeft < minTimeLeft )
	  minTimeLeft = intTimeLeft;
   }

   return minTimeLeft;
	}
//////////////////////////////////////////////////////////////////////////////
// UpdateAll:
//       updates all the auction in AucNumList, clycles throuh the list,
//      updates auctions or removes them if they are ended.  This 
//      is called by the long-cycle update timer.
// calls:
//      CmCancelClicked(), UpdateAuction(), DeleteAuction(), UpdateList()
//////////////////////////////////////////////////////////////////////////////
bool BidWatcher::UpdateAll(bool firstRun){
   int returnVal;
   int idx;
   char numerator[4];
   char denominator[4];
   char message[20];
   WriteAucFile();
   struct BidMakeup_type NewAuction;
   cancelButton->show();
   cancelPressed = FALSE;

   int prev_selected = aucList->currentItem();
   // loop through each auction, updating it as we hit it.
   for (idx=0;idx<aucIdx;idx++)
   {
       // make sure we don't interfere with a snipe...
       if( !firstRun && timeToNextEnd() < 5*60 )
       {
	   cancelPressed = FALSE;
	   cancelButton->hide();
	   return TRUE;
       }

      strcpy(message, " updating ");
      sprintf(denominator, "%d", aucIdx);
      sprintf(numerator, "%d", (idx + 1));
      strcat(message, numerator);
      strcat(message, "/");
      strcat(message, denominator);
      statLabel->setText(message);

      // if user pressed the cancel button, end the 
      // update right here.
      if ( cancelPressed == TRUE )
      {
	  statLabel->setText("");
	  cancelPressed = FALSE;
	  cancelButton->hide();
	  return FALSE;
      }
      // if the auction is over, we will delete it here, unless this is the
      // first run through.
      if ( (CalcTimeLeft(BidMakeup[idx].EndsValue, timeDiff) < AUTODEL) &&
             (BidMakeup[0].TimeLeft[0] != 0) && (autoDelete) )
      {   
         DeleteAuction(idx);
         aucList->removeItem(idx);
         idx--;
      }
      else
      {
         NewAuction = BidMakeup[idx];
         returnVal = UpdateAuction(NewAuction);

        if ( returnVal == 6 ) {
	 statLabel->setText(" Removing Invalid Auction...");   
         DeleteAuction(idx);
         aucList->removeItem(idx);
         idx--;  
        returnVal = 6;
        }

         if ( returnVal != 1 )
         {   
	     statLabel->setText("");
	     cancelPressed = FALSE;
	     cancelButton->hide();
	     return FALSE;
         }
         else
         {
            BidMakeup[idx] = NewAuction;
            UpdateListItem(idx);
         }
      }
   }
   aucList->setCurrentItem( prev_selected );
   aucList->centerCurrentItem();
   statLabel->setText("");
   cancelPressed = FALSE;
   cancelButton->hide();
   return FALSE;
	}
////////////////////////////////////////////////////////////////////////////// 
// doubleClick:
//       handler for doubleClick on a  listbox item.  Updates the 
//      items current info.
// calls:
//      UpdateAuction()
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::doubleClick(int clickIndex){
    launchBrowser( clickIndex );
	}
//////////////////////////////////////////////////////////////////////////////
// addNewItem:
//      called when user enters a number via numberEdit, does some error
//      checking on the supplied auction number, and if it looks valid, 
//      is submitted to the auction site for verification.
//   calls:
//      UpdateAuction(), DeleteAuction(), UpdateList()
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::addNewItem(char * newNumber){
   if ( updateInProgress == TRUE )
   {
      // over the limit on auctions already
      QMessageBox::about( this, "Error", "Program bug:  can't add auction while updating auctions." );
      return;
   }
   updateInProgress = TRUE;
   int    i;      // for counting
   int returnVal;
   char AuctionNum[30];
   // strip off leading whitespace
   while( *newNumber != '\0'
	  && (isspace( *newNumber ) || *newNumber == '=' || *newNumber == '#' )
	  )
       newNumber++;
   // strip off training whitespace
   for( i = strlen( newNumber ) - 1; i > 0; i-- )
   {
       if( isspace( newNumber[i] ) )
	   newNumber[i] = '\0';
       else
	   break;
   }
   
   if ( aucIdx >= MAXAUCS )
   {
      // over the limit on auctions already
      QMessageBox::about( this, "Error", "You already have the maximum allowed auctions." );
      updateInProgress = FALSE;
      return;
   }
   // if the auction number is less that 9 or more than 10, we KNOW
   // it's not an ebay number ( well, for now anyway)
   if ( ((strlen(newNumber)) < 9) || ((strlen(newNumber)) > 10) )
   {
      QMessageBox::about( this, "Error", "Invalid auction number.2" );
      updateInProgress = FALSE;
      return;
   }
   strcpy(AuctionNum, newNumber);
   // if AuctionNum has any non-digit chars, we KNOW it's not an ebay number
   for ( i=0; i<9; i++)
   {
      if ( !(isdigit(AuctionNum[i])) )
      {
         if ( AuctionNum[i] == '\0' ) break;
         QMessageBox::about(this,"Error","Invalid auction number.3");
         updateInProgress = FALSE;
         return;
      }
   }
   // scan current list, make sure this isn't a dup
   for ( i=0; i < aucIdx; i++)
   {
      if ( strcmp(AuctionNum, BidMakeup[i].ItemNumber) == 0 )
      {
         QMessageBox::about( this, "Error", "Auction is already on list." );
         updateInProgress = FALSE;
         return;
      }
   }
   // now we're reasonably sure that this is an ebay auction - add it to AucNumList, and submit to
   // UpdateAuction.  If it is a mistype or otherwise not valid, it will be caught there and deleted,
   // we will notify user.
   statLabel->setText(" adding auction...");
   struct BidMakeup_type NewAuction;
   ClearBidMakeUp(NewAuction);
   strcpy(NewAuction.ItemNumber, AuctionNum);
   returnVal = UpdateAuction( NewAuction );
   if ( returnVal == 1 )
   {
      NewAuction.EndsValue = CalcEndsValue(NewAuction.Ends);
      BidMakeup[aucIdx] = NewAuction;
      aucIdx++;
      statLabel->setText("");
      UpdateList();
      updateInProgress = FALSE;
      timer4Up();
      WriteAucFile();
      return;
   }
   else if ( returnVal == 3 || returnVal == 6 )
       QMessageBox::about(this,"Error","Invalid auction number.4");

   else if ( returnVal == 4 )
       QMessageBox::about(this,"Error","Connection time-out");

   else if ( returnVal != 10 )
       QMessageBox::about(this,"Error","Can't connect to server.");

   updateInProgress = FALSE;
   statLabel->setText("");
	}
//////////////////////////////////////////////////////////////////////////////
// timer1Up:
//      
// calls:
//      UpdateAuction()
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::timer1Up(){
   // this timer runs once on startup.
   updateInProgress = TRUE;
   TimeSync();
   if ( doStartup )
   {
      if ( GetUserBids() == 1 )
      {
         GetUserListings();
         UpdateList();
      }
      UpdateAll(TRUE);
   }
   updateInProgress = FALSE;
	}
	void BidWatcher::timer2Up()
	{
   static volatile int	checking_snipes = FALSE;

   if ( clockIsSet )
   {
      int timeLeft; 
      char newTime[10];
      int alarm_level = 0;
	#ifdef DEBUG_NETWORK
      static int prev_snipe_delay = 0;
	#endif


      int cur_snipe_delay = snipeDelay;
      int snipe_adj = (int)((avg_load_time - NORMAL_LOAD_TIME + 500)/1000);

      if( snipe_adj > 0 )
      {
	  cur_snipe_delay += snipe_adj;

	#ifdef DEBUG_NETWORK
	  if( prev_snipe_delay != cur_snipe_delay )
	  {
	      fprintf( stderr, "poor network response:  "
		       "Adjusting snipe delay from %d to %d\n",
		       snipeDelay, cur_snipe_delay );

	      prev_snipe_delay = cur_snipe_delay;
	  }
	#endif
      }
	#ifdef DEBUG_NETWORK
      else
      {
	  prev_snipe_delay = 0;
      }
	#endif


      for ( int i=0; i < aucIdx; i++ ) {
   
	  if ( BidMakeup[i].isSnipe ) {
	      timeLeft = CalcTimeLeft(BidMakeup[i].EndsValue, timeDiff);
	      
	      if ( !checking_snipes )
	      {
		  checking_snipes = TRUE;

		  if ( timeLeft < cur_snipe_delay ) {
		      BidMakeup[i].isSnipe = FALSE;
		      DoSnipe(i);
		  }

		  checking_snipes = FALSE;
	      }

	      if ( timeLeft > -5 )
	      {
		  if ( timeLeft < 120 )
		      alarm_level = 2;

		  else if ( timeLeft < 1800 )
		  {
		      if ( alarm_level < 1 )
			  alarm_level = 1;
		  }
	      }
	  }
      }

      if ( alarm_level == 0 )
	  timeLabel->setPalette(greenPalette);
      else if ( alarm_level == 1 )
	  timeLabel->setPalette(yellowPalette);
      else if ( alarm_level == 2 )
	  timeLabel->setPalette(redPalette);

      MakeClockTime(timeDiff, newTime);
      timeLabel->setText(newTime);

   }
   else
       timeLabel->setText( "no sync");
	}
//////////////////////////////////////////////////////////////////////////////
// bigUpdate
//      triggers an UpdateAll every BIGDELEY mSeconds. if there is an
//      update in progress, tries again in 2 minutes
//////////////////////////////////////////////////////////////////////////////
bool BidWatcher::bigUpdate(){
   if ( timeToNextEnd() < 5*60)
      return TRUE;

   TimeSync();
   if( timeToNextEnd() < 5*60 )
       return TRUE;
       
   if ( GetUserBids() == 1 )
   {
       if( timeToNextEnd() < 5*60 )
	   return TRUE;
       
      GetUserListings();
      if( timeToNextEnd() < 5*60 )
	  return TRUE;
       
      UpdateList();
      if( timeToNextEnd() < 5*60 )
	  return TRUE;
       
      if( UpdateAll(FALSE) )
	  return TRUE;
   }

   return FALSE;
	}
//////////////////////////////////////////////////////////////////////////////
// timer3up:
//      triggers an UpdateAll every BIGDELEY mSeconds. if there is an
//      update in progress, tries again in 2 minutes
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::timer3Up(){
   if ( updateInProgress == TRUE )
   {
      TIMER3->start(2*60*1000, TRUE);
      return;
   }

   updateInProgress = TRUE;
   if( bigUpdate() )
   {
       // update failed, try again soon
       TIMER3->start(5*60*1000, TRUE);
   }
   else
   {
       TIMER3->start(BIGDELAY, TRUE);    // long update
   }
   updateInProgress = FALSE;
	}
//////////////////////////////////////////////////////////////////////////////
// timer4Up - schedules the next update of the list of auctions
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::timer4Up(){
   int minTimeLeft;
   int time_to_next_update;
   int max_update;

   if ( aucIdx == 0 )
   {
       TIMER4->start(30*60*1000, TRUE);
       return;
   }

   minTimeLeft = timeToNextEnd() - (int)((avg_load_time + 500)/1000);

   if( minTimeLeft > -2 && minTimeLeft < snipeDelay + 2 )	// don't interfere with the snipe
   {
       TIMER4->start((minTimeLeft + 3) * 1000, TRUE);
       return;
   }

   if ( updateInProgress )
   {
       TIMER4->start(30*1000, TRUE);
       return;
   }
   updateInProgress = TRUE;
   UpdateLatest();
   updateInProgress = FALSE;

   minTimeLeft = timeToNextEnd() - (int)((avg_load_time + 500)/1000);

   if( minTimeLeft > 0 && minTimeLeft < snipeDelay + 2 )	// don't interfere with the snipe
       time_to_next_update = (minTimeLeft + 3) * 1000;
   else if( minTimeLeft < 3*60 )		// watch it even closer
       time_to_next_update = 10*1000;
   else if( minTimeLeft < 15*60 )		// watch it closely
       time_to_next_update = 60*1000;
   else if( minTimeLeft < 2*60*60 )		// no sense in wasting bandwidth...
       time_to_next_update = 5*60*1000;
   else						// *yawn*
       time_to_next_update = 15*60*1000;

   max_update = (int)(avg_load_time * 2);
   if( time_to_next_update < max_update )
   {

	#ifdef DEBUG_NETWORK
       fprintf( stderr, "poor network response:  "
		"Adjusting update time from %d to %d\n",
		time_to_next_update, max_update );
	#endif

       time_to_next_update = max_update;
   }
   
   TIMER4->start( time_to_next_update, TRUE );
	}
//////////////////////////////////////////////////////////////////////////////
// resizeEvent:
//      processes a window-resize event.
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::resizeEvent( QResizeEvent * ){
    //aucList->setGeometry( 0, 52, width()-1, height()-52 );
    aucList->setGeometry(0,44,width()-1, height()-44);
	}
//////////////////////////////////////////////////////////////////////////////
// ReadAucFile:
//      called once at startup, reads the username and stored auctions from
//      .aucFile
// calls:
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::ReadaucFile(){
   int rows;
   int dataID;
   char holder[200];
   char fileName[200];
   char * scratch;
   ifstream inFile;
   char * homedir = getenv("HOME");
   strcpy(fileName, homedir);
   strcat(fileName, "/.bidwatcher/bw.cfg");


   inFile.open(fileName, ios::in);
   
   if ( inFile )
   {
      inFile >> dataID;
      while ( inFile )
      {
         switch (dataID)
         {
            case 1: 
               inFile >> rows; // not used in linux version as of yet
               inFile >> soundOn;
               inFile >> trackListings;
               inFile >> trackBids;
               inFile >> autoDelete;
               inFile >> doStartup;
               inFile >> snipeDelay;
               inFile >> clockIsSet;
               if ( clockIsSet ) inFile >> timeDiff;
	       inFile >> earlycheck;
		inFile >> boldcheck;
		inFile >> fixedcheck;
		inFile >> fontSize;
               inFile.ignore(200,'\n');
               break;
            
           case 2:
               inFile.ignore();
               inFile.getline(authID, 76, '\n');
               break;
              
           case 3:
               inFile.ignore();
               inFile.getline(authPASS, 76, '\n');
               break;
             
            case 4:
               inFile.ignore();
               inFile.getline(browserPATH, 200, '\n');
               break;
            case 8:
               inFile.ignore();
               inFile.getline(emailPATH, 200, '\n'); 
               break;
	    case 10:
	       inFile.ignore();
	       inFile.getline(ebayPATH, 65, '\n');
	       break;
            case 11:
               inFile.ignore();
               inFile.getline(ebaymPATH, 65, '\n');
		break;
            case 5:
               if ( aucIdx < MAXAUCS )
               {
                  ClearBidMakeUp(BidMakeup[aucIdx]);
                  inFile.ignore();
                  inFile.getline(holder, 200, '\n');
                  scratch = strtok(holder, "\t");
                  if (scratch) strcpy(BidMakeup[aucIdx].ItemNumber, scratch);
                  scratch = strtok(0, "\n");
                  if (scratch) strcpy(BidMakeup[aucIdx].Description, scratch);
                  //
                  inFile >> dataID;
                  inFile >> BidMakeup[aucIdx].stat;
                  inFile >> BidMakeup[aucIdx].EndsValue;
                  inFile >> BidMakeup[aucIdx].isSnipe;
                  if ( BidMakeup[aucIdx].isSnipe )
                  {
                     inFile >> dataID;
                     inFile.ignore(); 
                     inFile.getline(holder,200, '\n');
                     scratch = strtok(holder, "\t");
                     if (scratch) strcpy(BidMakeup[aucIdx].snipeQty, scratch);
                     scratch = strtok(0,"\t");
                     if ( scratch ) strcpy(BidMakeup[aucIdx].snipeAmount, scratch);
                     scratch = strtok(0,"\n");
                     if ( scratch ) strcpy(BidMakeup[aucIdx].snipeKey, scratch);
                  }
                  aucIdx++;
               }  
               break;
           
           default:
              inFile.close();
              return;
         }
         inFile >> dataID;
      }
   }
   inFile.close();
   return;
	}
//////////////////////////////////////////////////////////////////////////////
// CmMenu
//    Produces a popup menu
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::CmMenu(){
   QPoint nowPos = QCursor::pos();
   listSelect->move(nowPos);
   listSelect->show();
	}
//////////////////////////////////////////////////////////////////////////////
// UpdateListItem:
//      replaces the item in the index position with current information.
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::UpdateListItem(int i){
   MyListBoxItem * newItem;
   char AucStat[250] = "";
   char chTemp[20];
   char * scratch;
   QColor theColor;

   int intTimeLeft = stringTimeLeft( BidMakeup[i].EndsValue, timeDiff, chTemp);
   if ( intTimeLeft < -30 )
       theColor = QColor(0,0,0);
   else if ( intTimeLeft < 14400 )
      theColor = QColor(200,0,0);
   else theColor = QColor(0,0,255);
   // we have 3 possible states:
   // 1. info from saved file
   // 2. info from an auction list grab
   // 3. all info
   int infoState = 1;
   if ( BidMakeup[i].CurrentBid[0] != 0 ) {
      if ( BidMakeup[i].TimeLeft[0] != 0 ) infoState = 3;
      else infoState = 2;
   }   

   strcpy(AucStat, BidMakeup[i].ItemNumber);
   strcat(AucStat, "   ");
   if ( infoState == 1 ) {
	strcat(AucStat, " -             ");
   }
   if ( infoState == 2 ) {
      strcat(AucStat, BidMakeup[i].CurrentBid);
     while (strlen(AucStat) < 27 ) strcat(AucStat, "   ");
    int a1=27, c1=strlen(AucStat);
    while (c1 < a1) {                     /* adds spaces */
     strcat(AucStat, " ");   
          c1++;
    }

   }
   else if ( infoState == 3 )
   {
         strcat(AucStat, BidMakeup[i].CurrentBid);
         strcat(AucStat, "(");
         strcat(AucStat, BidMakeup[i].BidCount);
     if( BidMakeup[i].reserveMet == 'n' && BidMakeup[i].BuyitNow == 'y')
          strcat(AucStat, "RB" );
         if( BidMakeup[i].reserveMet == 'n' && BidMakeup[i].BuyitNow == 'n')
             strcat(AucStat, "R" );
      if( BidMakeup[i].BuyitNow == 'y' && BidMakeup[i].reserveMet == 'x')
          strcat(AucStat, "B" );
         if ( strcmp(BidMakeup[i].HighBidder, "Dutch Auction") == 0)
             strcat(AucStat, "D" );
         strcat(AucStat, ")");

    int a1=28, c1=strlen(AucStat);
    while (c1 < a1) {                     /* adds spaces */
     strcat(AucStat, " ");
          c1++;
    }
      }

    strcat(AucStat, /*BidMakeup[i].TimeLeft*/ chTemp);
    int a1=45, c1=strlen(AucStat);
    while (c1 < a1) {                     /* adds spaces */
     strcat(AucStat, " ");
          c1++;
    }

     if( BidMakeup[i].snipeAmount[0]==0) {
     scratch = strstr(BidMakeup[i].BuyitNowPrice, "Sold!");
	if (scratch != NULL ) {
	theColor = QColor(0,0,0);
	strcat(AucStat, BidMakeup[i].BuyitNowPrice);
	}
	scratch = strstr(BidMakeup[i].BuyitNowPrice, ".");
	if (scratch == NULL ) {
	}
	else
	{	
	strcat(AucStat, BidMakeup[i].BuyitNowPrice);
	}
	}
	else
	{

	if (BidMakeup[i].isSnipe) {
   if ( intTimeLeft < 14400 )
      theColor = QColor(255,50,50);
   else theColor = QColor(0,175,50);

	if (BidMakeup[i].BuyitNow == 'y') {
	strcat(AucStat, BidMakeup[i].BuyitNowPrice);
	strcat(AucStat," ");
	}
	strcat(AucStat, BidMakeup[i].currency);
	strcat(AucStat, BidMakeup[i].snipeAmount);
	}
	}

	if (BidMakeup[i].endedNow == 1) {
	theColor = QColor(0,0,0);
	strcat(AucStat,"Seller Ended");
	}
    int aa1=64, cc1=strlen(AucStat);
    while (cc1 < aa1) {                     /* adds spaces */  
     strcat(AucStat, " ");
          cc1++;
    }

   strcat(AucStat, BidMakeup[i].Description);
   if ( BidMakeup[i].isSnipe ) {
    newItem = new MyListBoxItem(AucStat, snipeDot, theColor );
    aucList->changeItem(newItem, i);
   }
   else
   {
      if ( BidMakeup[i].stat == '1' ) {
    newItem = new MyListBoxItem( AucStat, redDot, theColor );
   }
   else
   {
      if ( BidMakeup[i].stat == '2' ) newItem = new  MyListBoxItem(AucStat, greenDot, theColor);
      else if ( BidMakeup[i].stat == '3' ) newItem = new  MyListBoxItem(AucStat, blueDot, theColor);
      else newItem = new MyListBoxItem(AucStat, blackDot, theColor);
   }
   aucList->changeItem(newItem, i);
	}
	 }
//////////////////////////////////////////////////////////////////////////////
// DeleteAuction:
//      deletes the auction at position index, adjust both the AucNumList[]
//      and BidMakeup[] to fill the gap created.
// calls:
//      ClearBidMakeup()
//////////////////////////////////////////////////////////////////////////////   
void BidWatcher::DeleteAuction(int index ){
   int idx; 
   if ( index >= aucIdx ) return;  // we are out of range
   // Loop through AucNumList, moving list up to fill the gap
   // where choice will be removed, this should be a linked list.  
   WriteLog(index); 
   for(idx = index ; idx < (aucIdx - 1 ); idx++) 
   {
      ClearBidMakeUp(BidMakeup[idx]);
      BidMakeup[idx] = BidMakeup[idx+1];
   }
   ClearBidMakeUp(BidMakeup[aucIdx - 1]);
   aucIdx--; 
	}
//////////////////////////////////////////////////////////////////////////////
//
// GetUserAuctions - grabs users auctions from ebay and adds them to
//    aucNum list if they aren't already there.
//   return -2 if userID is not recognized by ebay
//  return -1 if grab otherwise fails (lost connection, etc)
//   return -5 if the cgi server is down
//  return -4 on timeout
//  retunrn 1 on success
//////////////////////////////////////////////////////////////////////////////
int BidWatcher::GetUserBids(){
   bool flagMatch;
   struct BidMakeup_type TempList[MAXAUCS + 1]; 
   int listCount = 0;
   int idx, cnt;
   char Buff[HUGEBUFF];
   char Buff2[HUGEBUFF];
   char WebPage[200];
   int returnVal;
   strcpy(WebPage, "/aw-cgi/eBayISAPI.dll?MfcISAPICommand=ViewBidItems&userid=");
   strcat(WebPage, authID);
   strcat(WebPage, "&completed=0&all=1&rows=200");

   if ( strlen(authID) < 2 ) return -1;
   if ( strlen(authID) > 64 ) return -2;
   
   statLabel->setText(" getting bids");
   brightLabel->setPixmap(brightLed);
   returnVal = FetchHtml(WebPage, Buff, theAddress3, HUGEBUFF, TIMEOUT );
   brightLabel->setPixmap(dimLed);
   statLabel->setText("");
   if ( (returnVal == 2) || (returnVal == 10) || (returnVal == 4) ) {
	return -1;
	}
   if ( strstr(Buff, "invalid") != NULL ) {
	return -2;
	}
   parseRows(Buff, Buff2);
   StripAndTab(Buff2, Buff, HUGEBUFF);
   listCount = ParseList( TempList, Buff );
   if ( listCount < 0 ) {
	return -1;
	}
   // copy TempList to bidlist and set the number of user bids
   // that we have on store from ebay
   for ( cnt=0; cnt < listCount; cnt++ )
   {
      strcpy(BidList[cnt], TempList[cnt].ItemNumber);
   }
   if ( !trackBids ) return 1;
   bidIdx = listCount;
   // now, compare each auction to the current auction list,
   // if there are no matches, insert it into BidMakeup.
   // also check for matches that are not flagged as owned by
   // user (this could happen if the auction was entered manually
   // before the user bid on the auction ).
   for ( cnt=0; cnt < listCount; cnt++)
   {
      if ( aucIdx >= MAXAUCS ) return 1;
      flagMatch = FALSE;
      for ( idx=0; idx < aucIdx; idx++ )
      {
         if ( strcmp(BidMakeup[idx].ItemNumber, TempList[cnt].ItemNumber) == 0 )
         {
	    ClearBidMakeUp(BidMakeup[aucIdx]);
            strcpy(BidMakeup[idx].CurrentBid, TempList[cnt].CurrentBid);
	    strip_commas( BidMakeup[idx].CurrentBid );
	    BidMakeup[aucIdx].EndsValue = CalcEndsValue(TempList[cnt].Ends);
	    translate_date( TempList[cnt].Ends );
	    strcpy(BidMakeup[aucIdx].Ends, TempList[cnt].Ends);
	    strcpy(BidMakeup[aucIdx].Description, TempList[cnt].Description);
	    BidMakeup[aucIdx].stat = '0';
            flagMatch = TRUE;
         }
      }
      if ( flagMatch == FALSE )
      {
         ClearBidMakeUp(BidMakeup[aucIdx]);
         strcpy(BidMakeup[aucIdx].Ends, TempList[cnt].Ends);
	 translate_date( TempList[cnt].Ends );
         strcpy(BidMakeup[aucIdx].CurrentBid, TempList[cnt].CurrentBid);
	 strip_commas( BidMakeup[aucIdx].CurrentBid );
         BidMakeup[aucIdx].EndsValue = CalcEndsValue(TempList[cnt].Ends);
         strcpy(BidMakeup[aucIdx].ItemNumber, TempList[cnt].ItemNumber);
         strcpy(BidMakeup[aucIdx].Description, TempList[cnt].Description);
         BidMakeup[aucIdx].stat = '3';
         aucIdx++;

      }
   }
   return 1;
	} 
// end GetUserAuctions.
/////////////////////////////////////////////////////////////////////////////
// GetUserListings
/////////////////////////////////////////////////////////////////////////////
int BidWatcher::GetUserListings(){
   if ( !trackListings ) return 1;
   int idx;
   char Buff[HUGEBUFF];
   char Buff2[HUGEBUFF];
   char WebPage[200];

   int returnVal;
   strcpy(WebPage, "/aw-cgi/eBayISAPI.dll?ViewListedItems&userid=");
   strcat(WebPage, authID);
   strcat(WebPage, "&completed=0&sort=3&since=-1");
   if ( strlen(authID) < 2 ) return -1;
   if ( strlen(authID) > 64 ) return -2;
   statLabel->setText(" getting listings");
   brightLabel->setPixmap(brightLed);
   returnVal = FetchHtml(WebPage, Buff, theAddress6, HUGEBUFF, TIMEOUT);
   brightLabel->setPixmap(dimLed);
   statLabel->setText("");

   if ( (returnVal == 2) || (returnVal == 10) || (returnVal == 4) ) return -1;

   if ( strstr(Buff, "invalid") != NULL ) return -2;



   parseRows2(Buff, Buff2);
        istrstream streamBuff(Buff2);
	char lineBuff[12];
	bool adOK=FALSE;

        while ( streamBuff ) {
                streamBuff.getline(lineBuff, 11, '\n'); 
	if (strlen(lineBuff) < 10 ) return 1;
		updateInProgress = FALSE;
      for ( idx=0; idx <= aucIdx; idx++ )
      {
	if ( strcmp(BidMakeup[idx].ItemNumber, lineBuff) == 0 ) adOK = TRUE; 
	}
	if ( adOK != 1 ) addNewItem(lineBuff);
	adOK=FALSE;
	}
   return 1;
	}
///////////////////////////////////////////////////////////////////
///////////------------------- UpdateList -------------////////////
///////////////////////////////////////////////////////////////////
void BidWatcher::UpdateList(){
   MyListBoxItem * newItem;
   char AucStat[250] = "";
   char * scratch;
   char chTemp[20];
   ArrangeList();
   int prev_selected = aucList->currentItem();
   aucList->clear();
   aucList->hide();
   QColor theColor;
   int intTimeLeft;
   int infoState;
   for( int i = 0; i < aucIdx; i++ )
   {
      infoState = 1;
      intTimeLeft = stringTimeLeft( BidMakeup[i].EndsValue, timeDiff, chTemp);
      if ( intTimeLeft < -30 )
	  theColor = QColor(0,0,0);
      else if ( intTimeLeft < 14400 )
         theColor = QColor(200,0,0);
      else theColor = QColor(0,0,255);
   // we have 3 possible states:
   // 1. info from saved file
   // 2. info from an auction list grab
   // 3. all info
      if ( BidMakeup[i].CurrentBid[0] != 0 ) {
         if ( BidMakeup[i].TimeLeft[0] != 0 ) infoState = 3;
         else infoState = 2;
      }   
      strcpy(AucStat, BidMakeup[i].ItemNumber);
      strcat(AucStat, "   ");
      if ( infoState == 1 ) {
         strcat(AucStat, " -             ");
      }
      if ( infoState == 2 ) {
         strcat(AucStat, BidMakeup[i].CurrentBid);
    int a1=28, c1=strlen(AucStat);
    while (c1 < a1) {                     /* adds spaces */
     strcat(AucStat, " ");
          c1++;
    }
      }
      else if ( infoState == 3 )
      {
         strcat(AucStat, BidMakeup[i].CurrentBid);
         strcat(AucStat, "(");
         strcat(AucStat, BidMakeup[i].BidCount);
     if( BidMakeup[i].reserveMet == 'n' && BidMakeup[i].BuyitNow == 'y')
          strcat(AucStat, "RB" );
	 if( BidMakeup[i].reserveMet == 'n' && BidMakeup[i].BuyitNow == 'n')
	     strcat(AucStat, "R" );
      if( BidMakeup[i].BuyitNow == 'y' && BidMakeup[i].reserveMet == 'x')
          strcat(AucStat, "B" );
	 if ( strcmp(BidMakeup[i].HighBidder, "Dutch Auction") == 0)
	     strcat(AucStat, "D" );
         strcat(AucStat, ")");
    int a1=28, c1=strlen(AucStat);
    while (c1 < a1) {                     /* adds spaces */  
     strcat(AucStat, " ");
          c1++;
    }
      }
      strcat(AucStat, /*BidMakeup[i].TimeLeft*/ chTemp);
    int a1=45, c1=strlen(AucStat);
    while (c1 < a1) {                     /* adds spaces */
     strcat(AucStat, " ");
          c1++;
    }
     if( BidMakeup[i].snipeAmount[0]==0) {
     scratch = strstr(BidMakeup[i].BuyitNowPrice, "Sold!");
        if (scratch != NULL ) {
        theColor = QColor(0,0,0);
        strcat(AucStat, BidMakeup[i].BuyitNowPrice);
        }
        scratch = strstr(BidMakeup[i].BuyitNowPrice, ".");   
        if (scratch == NULL ) {
        }
        else
        {
        strcat(AucStat, BidMakeup[i].BuyitNowPrice);
        }
        }
        else
        {
        if (BidMakeup[i].isSnipe) {
   if ( intTimeLeft < 14400 )
      theColor = QColor(255,50,50);
   else theColor = QColor(0,175,50);
        if (BidMakeup[i].BuyitNow == 'y') {
        strcat(AucStat, BidMakeup[i].BuyitNowPrice);
        strcat(AucStat," ");
        }

        strcat(AucStat, BidMakeup[i].currency);
        strcat(AucStat, BidMakeup[i].snipeAmount);
        }
        }

	if (BidMakeup[i].endedNow == 1) {
        theColor = QColor(0,0,0);
        strcat(AucStat,"Seller Ended");
        }

    int aa1=64, cc1=strlen(AucStat);
    while (cc1 < aa1) {                     /* adds spaces */  
     strcat(AucStat, " ");
          cc1++;
    }
      strcat(AucStat, BidMakeup[i].Description);
      if ( BidMakeup[i].isSnipe ) newItem = new MyListBoxItem( AucStat, snipeDot,theColor);
      else
      {
         if ( BidMakeup[i].stat == '1') newItem = new MyListBoxItem( AucStat, redDot,theColor );
         else if ( BidMakeup[i].stat == '2' ) newItem = new  MyListBoxItem(AucStat, greenDot,theColor);
         else if ( BidMakeup[i].stat == '3' ) newItem = new  MyListBoxItem(AucStat, blueDot,theColor);
         else newItem = new MyListBoxItem(AucStat, blackDot,theColor);
      }
      aucList->insertItem(newItem, i);
   }
   aucList->setCurrentItem(prev_selected);
   aucList->show();
   return;
	}   
//////////////////////////////////////////////////////////////////////////////
// ArrangeList
//
// arrange the list in order of auction ending time.
//////////////////////////////////////////////////////////////////////////////
void BidWatcher::ArrangeList(){
   struct BidMakeup_type Scratch;
   int cnt = 0;
   bool swapflag = TRUE;

   while ( swapflag ) {
     swapflag = FALSE;
     for (cnt=0; cnt < (aucIdx - 1); cnt++) {
         if ( BidMakeup[cnt].EndsValue > BidMakeup[cnt + 1].EndsValue )  {
            Scratch = BidMakeup[cnt];
            BidMakeup[cnt] = BidMakeup[cnt + 1];
            BidMakeup[cnt + 1] = Scratch;
            swapflag = TRUE;
         }
      }
   }
 }      
//////////////////////////////////////////////////////////////////////////////
// Updates the auction information for the AucListNum of the
// index number passed.
//      Returns 1 success
//      Returns 2 connection problem
//      Returns 3 invalid auction (was deleted)
//      Returns 4 timeout
//      Returns 5 ebay cgi is down
//      Returns 6 itemNumber is out of bounds.
//      Returns 10 if the update was user-cancelled
//////////////////////////////////////////////////////////////////////////////
int BidWatcher::UpdateAuction( struct BidMakeup_type &  NewAuction){
   int returnVal;
   char HtmlBuff[HUGEBUFF];
   char WebPage[80];
   strcpy(WebPage,ebayPATH);
   strcat(WebPage,"?ViewItem&item=");
   strcat(WebPage, NewAuction.ItemNumber);
   brightLabel->setPixmap(brightLed);
   returnVal = FetchHtml(WebPage,HtmlBuff,theAddress, HUGEBUFF, TIMEOUT);

   brightLabel->setPixmap(dimLed);
   if ( returnVal == 10 ) return 10;
   else if ( returnVal == 2 ) return 2;
   else if ( returnVal == 4 ) return 4;
        if ( strlen(HtmlBuff) < 1000 ) {
	return 5;
	}

   else if ( strlen(HtmlBuff) < 5000 ) return 3;
   // ok, html grab was a success - now process the html

   char Buff[HUGEBUFF];
   StripHtmlTags(HtmlBuff, Buff, HUGEBUFF);
   if ( GetAucInfo(Buff, NewAuction) ) {
      // if the grab was a success, check to see if it is a user 
      // called auction.

      if ( strcmp(NewAuction.Seller, authID) == 0 ) {
         NewAuction.stat = '1';
      }
      else
      {

          if ( strcmp(NewAuction.HighBidder, authID) == 0 ) {
             NewAuction.stat = '2';
          }
          else
          {
  if ( (NewAuction.stat == '2') && (strcmp(NewAuction.HighBidder, authID) != 0) ) NewAuction.stat = '3';
        int bidIdx = aucIdx;
             for ( int i2=0; i2 < bidIdx; i2++ )
             {

              if ( strcmp(NewAuction.ItemNumber, BidList[i2]) == 0 ) {
                   NewAuction.stat = '3';
                }
             }
          }
       }
       return 1;
    }
    else 
	return 6;
	}   
///////////////////////////////////////////////////////////////////////////////
//  TimeSync
///////////////////////////////////////////////////////////////////////////////
void BidWatcher::TimeSync(){
   statLabel->setText(" syncing clock");
   int returnVal, i, tDiff;
   char HtmlBuff[HUGEBUFF];
   char WebPage[45];
   int		new_timeDiff = -24*60*60;
   struct timeval tm_start, tm_end;
   double	t1;


   strcpy(WebPage, "/aw-cgi/eBayISAPI.dll?TimeShow");
   
   // try getting the time several times and pick the minimum time
   // difference.  This should reduce the variable network delay.  

   int num_good = 0;

   for( i = 0; i < 10 && num_good < 3; i++ )
   {
       brightLabel->setPixmap(brightLed);
       gettimeofday( &tm_start, NULL );

       returnVal = FetchHtml(WebPage, HtmlBuff, theAddress3, HUGEBUFF,
			     TIMEOUT);

       gettimeofday( &tm_end, NULL );
       t1 = (tm_end.tv_sec + 0.000001 * tm_end.tv_usec)
	   - (tm_start.tv_sec + 0.000001 * tm_start.tv_usec);

       brightLabel->setPixmap(dimLed);

       if( t1 > 6 )
       {

	#ifdef DEBUG_NETWORK
	   fprintf( stderr, "Took too long (%.2f sec) to fetch ebay's time web page, retrying... (%d)\n", t1, i );
	#endif

	   continue;
       }

       if ( strlen(HtmlBuff) < 1000 )
       {

	#ifdef DEBUG_NETWORK
	   fprintf( stderr, "Short web page (%d bytes) from ebay's time web page, retrying... (%d)\n", strlen( HtmlBuff ), i );
	#endif

	   continue;
       }

       if ( returnVal != 1 )
       {

	#ifdef DEBUG_NETWORK
	   fprintf( stderr, "Error (%d) fetching ebay's time web page, retrying... (%d)\n", returnVal, i );
	#endif

	   continue;
       }
       
       tDiff = calcTimeDiff(HtmlBuff);
       // Network delays can cause the time returned by ebay to be
       // off, so we try to correct for this error by removing half the
       // total packet time.  The time returned by ebay is (most likely)
       // truncated to the nearest second.  We try to correct for this by
       // adding in a half second to "round" it correctly.

	#ifdef DEBUG_NETWORK
       fprintf( stderr, "Adjusting tDiff from %d by %.2f\n",
		tDiff, t1/2 + .5 );
	#endif

       tDiff += (int)(t1/2 + .5);

       // is the time difference total garbage?
       if( tDiff < -24*60*60 || tDiff > 24*60*60 )
       {

	#ifdef DEBUG_NETWORK
	   fprintf( stderr, "Bogus tDiff (%d) from ebay's time web page, retrying... (%d)\n", tDiff, i );
	#endif

	   continue;
       }

       if( tDiff > new_timeDiff )
	   new_timeDiff = tDiff;

       num_good++;
   }


	#ifdef DEBUG_NETWORK
   if( num_good != i )
       fprintf( stderr,
		"Failed to get a good sync %d out of %d times\n",
		i - num_good, i );
	#endif

   time_t time_now = time(0);
   int	gmt_time;
   int		local_timeDiff;

	#if defined(HAVE_GMTIME)
	gmt_time = time_now - mktime(gmtime(&time_now));
	#elif defined(HAVE_TIMEZONE)
	struct tm * timeStruct = localtime(&time_now);	// just to set timezone global variable
	gmt_time = -timezone;
	#elif defined(HAVE_GMTOFF)
	struct tm * timeStruct = localtime(&time_now);
	gmt_time = timeStruct->tm_gmtoff;
	if( timeStruct->tm_isdst )
	gmt_time -= 60*60;	// correct for daylight savings time
	#else
	#error "One of -DHAVE_GMTIME -DHAVE_TIMEZONE or -DHAVE_GMTOFF must be defined"
	#endif

   // ebay is GMT+8hrs
   local_timeDiff = -8*60*60 - gmt_time;  // get real diff

   static bool warned_timeDiff = FALSE;
   if( num_good > 0 && !warned_timeDiff
       && abs( local_timeDiff - new_timeDiff ) > 15*60 )
   {
       char	msg[512];
       sprintf( msg,
		"Using the local clock and timezone information to calculate\n"
		"eBay's time is %d seconds different than the time returned\n"
		"from eBay's web site.  This may be a problem.",
		local_timeDiff - new_timeDiff );
       
       QMessageBox::about(this,"Warning", msg );
       warned_timeDiff = TRUE;
   }

   if ( num_good < 2 )
   {
       clockIsSet = TRUE;
       timeDiff = local_timeDiff;

       // ebay is GMT+8hrs, but add a 5 minute fudge so that we bid
       // early to be safe
       
       timeDiff += 5*60;
	   
	#ifdef DEBUG_NETWORK
       fprintf( stderr,
		"Could not sync time with Ebay, using local clock." );
	#endif

       QMessageBox::about(this,"Error",
			  "Could not sync time with ebay, using local clock.");
   }
   else
   {

	#ifdef DEBUG_NETWORK
       if( clockIsSet && abs( new_timeDiff - timeDiff ) > 2 )
	   fprintf( stderr, "Unusually large time change:  %d - %d = %d\n",
		    new_timeDiff, timeDiff, new_timeDiff - timeDiff );
	#endif

       clockIsSet = TRUE;
       timeDiff = new_timeDiff;
   }

	#ifdef DEBUG_NETWORK
   fprintf( stderr, "TimeSync()  time diff=%d   local time diff=%d\n",
	    timeDiff, local_timeDiff );
	#endif

   statLabel->setText("");
	}
/////////////////////////////////////////////////////////////////////////////
// setupSnipe
/////////////////////////////////////////////////////////////////////////////
void BidWatcher::setupSnipe(struct BidMakeup_type & snipeAuction, int idx){
   BidMakeup[idx] = snipeAuction;
   UpdateListItem(idx);
   WriteAucFile();
	}
/////////////////////////////////////////////////////////////////////////////
// WriteLog - append BidMakeup[index] to the completed auction log.
/////////////////////////////////////////////////////////////////////////////
void BidWatcher::WriteLog( int index ){
   if ((BidMakeup[index].TimeLeft[0] != '0') && 
       (CalcTimeLeft(BidMakeup[index].EndsValue, timeDiff) < 0) )
   {
      ofstream reportOut;
      char * homedir = getenv("HOME");
      char fileName[200];
      MakeFileName(homedir, "/.bidwatcher/log", fileName, TRUE);
      reportOut.open(fileName, ios::app);
      char truncDesc[300];
      strcpy( truncDesc,   BidMakeup[index].ItemNumber);
      strcat( truncDesc, " - ");
      strcat( truncDesc, BidMakeup[index].Description );
      strcat( truncDesc, "\nHigh Bidder: ");
      strcat( truncDesc, BidMakeup[index].HighBidder);
      strcat( truncDesc, "\nHigh Bid   : ");
      strcat( truncDesc, BidMakeup[index].CurrentBid);
      strcat( truncDesc, "\n-----------------------------------------------------\n");
      reportOut << truncDesc;
      reportOut.close();
      ClearSnipe(index);
   }
 }
///////////////////////////////////////////////////////////////////////////////////
///// DoSnipe()
///////////////////////////////////////////////////////////////////////////////////
void BidWatcher::DoSnipe(int index){
   char Buff[HUGEBUFF];
   char WebPage[200];
   char startTime[10];
   char finishTime[10];


   // now, submit the bid, then check the resulting buffer
   // for a message indicating it was successful:
   MakeClockTime(timeDiff, startTime);
   strcpy(WebPage, "/aw-cgi/eBayISAPI.dll?AcceptBid&item=");
   strcat(WebPage, BidMakeup[index].ItemNumber);
   strcat(WebPage, "&key=");
   strcat(WebPage, BidMakeup[index].snipeKey);
   strcat(WebPage, "&userid=");
   strcat(WebPage, authID);
   strcat(WebPage, "&pass=");
   strcat(WebPage, authPASS);
   strcat(WebPage, "&maxbid=");
   strcat(WebPage,  BidMakeup[index].snipeAmount);
   strcat(WebPage, "&quant=");
   strcat(WebPage, BidMakeup[index].snipeQty);
   ofstream reportOut;
   char * homedir = getenv("HOME");
   char fileName[200];
   MakeFileName(homedir, "/.bidwatcher/snipe", fileName, TRUE);
   reportOut.open(fileName, ios::app);
   char truncDesc[1024];
   sprintf( truncDesc, "%s $%s %s",
	    BidMakeup[index].ItemNumber,
	    BidMakeup[index].snipeAmount,
	    BidMakeup[index].Description );
   ClearSnipe(index);

   brightLabel->setPixmap(brightLed);
   int returnVal = FetchHtml(WebPage, Buff, theAddress, HUGEBUFF, TIMEOUT);
   brightLabel->setPixmap(dimLed);

   if( returnVal != 1 )
   {
       reportOut << "Bid processing failed:  error" << returnVal << "\n";
       returnVal = 6;
   }
   else
       returnVal = ProcessBid(Buff);

   if( returnVal == 6 )
   {
       reportOut << Buff;

       brightLabel->setPixmap(brightLed);
       returnVal = FetchHtml(WebPage, Buff, theAddress, HUGEBUFF, TIMEOUT);
       brightLabel->setPixmap(dimLed);

       returnVal = ProcessBid(Buff);
  }

   MakeClockTime(timeDiff, finishTime);
   strcat(truncDesc, "\nSnipe start: " );
   strcat(truncDesc, startTime );
   strcat(truncDesc, "   finish: " );
   strcat(truncDesc, finishTime );
   strcat(truncDesc, "   auction ends: " );
   strcat(truncDesc, BidMakeup[index].Ends );

   BidMakeup[index].stat = '3';

   if ( returnVal == 1 ) {
      strcat(truncDesc, "\nSnipe completed: you are the high bidder!");
      strcat(truncDesc, "\n---------------------------------------------------\n");
      strcpy(BidList[bidIdx], BidMakeup[index].ItemNumber);
      BidMakeup[index].stat = '2';
      bidIdx++;
   }
   else if ( returnVal == 2 ) {
      strcat(truncDesc, "\nSnipe completed: but you were outbid");
      strcat(truncDesc, "\n---------------------------------------------------\n");
   }
   else if ( returnVal == 3 ) {
      strcat(truncDesc, "\nSnipe failed: bid under the current asking price");
      strcat(truncDesc, "\n---------------------------------------------------\n");
   }
   else if ( returnVal == 4 ) {
      strcat(truncDesc, "\nSnipe failed: problem with bid quantity");
      strcat(truncDesc, "\n---------------------------------------------------\n");
   }
   else if ( returnVal == 5 ) {
      strcat(truncDesc, "\nSnipe failed: auction ended before bid placed");
      strcat(truncDesc, "\n---------------------------------------------------\n");
   }
   else if ( returnVal == 6 ) {
      strcat(truncDesc, "\nSnipe failed: invalid username or password");
      strcat(truncDesc, "\n---------------------------------------------------\n");
   }
   else {
      strcat(truncDesc, "\nSnipe failed: unknown reason");
      strcat(truncDesc, "\n---------------------------------------------------\n");

	#if DEBUG_NETWORK
      ofstream outFile;
      char fileName[200];
      sprintf( fileName, "%s%s/snipe_fail_%s",
	       getenv("HOME"),"/.bidwatcher/", BidMakeup[index].ItemNumber );

      outFile.open(fileName);
      if ( outFile )
	  outFile << Buff;
      outFile.close();
	#endif

   }
   reportOut << truncDesc;
   reportOut.close();
   WriteAucFile();
   return;
	}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////// ----------- end BidWatcher class---------////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
int main( int argc, char ** argv ){
    QApplication a( argc, argv );
    BidWatcher m;

    a.setMainWidget( &m );
    m.show();
    return a.exec();
 }
